//2013.01.17 The UGen Classes & {Function}.play
//Written by Eli Fieldsteel <eli.fieldsteel@gmail.com>

//BOOTING SCSYNTH

//In order to produce audio, the scsynth application must be booted. This can be done by evaluating the following:

Server.local.boot;

//Alternatively, SC stores Server.local in the global variable 's', so the following should work just as well:

s.boot;

//You can also view or change the keyboard shortcut for this action in the preferences dialog.

//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////

//{FUNCTION}.PLAY

//"play" is a convenience method defined for Functions (among other classes):
{SinOsc.ar}.play;

// To stop all signal processing and wipe all sound entities from the server, press [cmd]+[period].

// Alternatively, you can evaluate the following:
Server.local.freeAll;

//or
s.freeAll;

//Notice that when we play a function, a Synth is returned in the post window. Synths can be "freed" individually, if you store your synth using a variable:
x = {SinOsc.ar}.play;

x.free;

//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////

//UGENS

SinOsc is one of the many subclasses of UGen, the basic signal generator in SC, equivalent to CSound opcodes (e.g. oscili) or MaxMSP tilde objects (e.g. [cycle~]). You can manually browse UGens in the help documentation, or search for the help file called "Tour of UGens". Nearly every UGen helpfile includes short {function}.play examples, which you can evaluate from within the help file itself.

UGens fall into a number of different categories. Here are some of the more common categories:

/*
Oscillators (Deterministic Generators)
Noise Generators (Stochastic Generators)
Envelopes
Triggers
Buffer Recording & Playback
Filters
Input/Output

...and so many more
*/

//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////

//CLASS METHODS

Although many instances of objects are generated using the "new" method (e.g. Rect.new, Window.new, Array.new, etc), UGens are instantiated using the following methods:

.ar	(audio rate)
.kr (control rate)
.ir (initialization rate)

//For example, the following produces an audio rate oscillator:
SinOsc.ar

//The following produces a control rate sine wave oscillator:
SinOsc.kr

//By default, SC's block size is 64 samples, which means an audio rate UGen will output 64 samples per block/control cycle, while a control rate UGen will output 1 sample per block/control cycle.

//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////

//ARGUMENTS

Different UGens expect different arguments (e.g. frequency, phase, width, etc). The SC environment will display the arguments, in order, with their default values, after typing the initial left parenthesis:


//If you type this line yourself, you should see a small in-editor pop-up after typing the first parenthesis. This pop-up shows the default argument names and values.
SinOsc.ar();

//You should see a small box that says "freq = 440, phase = 0, mul = 1, add = 0." Nearly every UGen allows for 'mul' and 'add' as a final pair of arguments. These values will be multiplied and added to the UGen's output values, in that order. 'mul' and 'add' are usually used to scale/shift the signal output.

//The following produces a sine wave at 300Hz, with an initial phase offset of pi, at half-normalized output, with no DC offset.
x = {SinOsc.ar(300, pi, 0.5, 0)}.play;
x.free;

//Arguments can be bypassed by specifying them explicitly. In the following case, we skip over phase (and therefore use the default value) and specify a mul value of 0.2. We also do not specify 'add', and therefore use its default value (zero) as well:
x = {SinOsc.ar(300, mul:0.2)}.play;
x.free;

//This is also fine, but unnecessary since 'freq' is the first argument:
x = {SinOsc.ar(freq:300, mul:0.2)}.play;
x.free;

//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////

//MORE COMPLEX UGEN FUNCTIONS

As was the case with ordinary Functions, we can specify variables, as well as arguments to be passed into the function upon evaluation. Remember that you can double-click on enclosure braces to select their contents for ease of evaluation.

(
f = {
	arg freq=200, amp=0.2, duty=0.5;
	var sig;

	//output a simple pulse wave
	sig = Pulse.ar(freq, duty, amp);
};
)

// Instantiate and free Synths using this function while supplying a variety of values for the arguments we've defined:

x = f.play; //default values
x.free;

//Notice that the order in which the arguments are declared does not have to match the order in which they are subsequently specified.
x = f.play(args: [\duty, 0.01, \freq, 40]);
x.free;

(
x = f.play(
	args: [
		\freq, 1600,
		\amp, 0.08,
		\duty, 0.33
]);
)

x.free;


UGens can be used to control other UGens. In this case, we are no longer specifying numeric values for our pulse wave's parameters, but instead these parameters are being controlled by other UGens. This is where the real fun begins.

The following example also makes use of the "range" and "exprange" methods, which are defined for all UGens. This is a more intuitive alternative to rescaling UGens via 'mul' and 'add' arguments.

IMPORANT!! Do not use a range/exprange method while also specifying 'mul' and 'add'; conflicts will arise. Use one or the other.

(
f = {
	var sig, freq, duty, amp;

	//linearly interpolated noise generator,
	//generating a new value 20x per sec
	amp = LFNoise1.kr(20).exprange(0.02,0.5);

	//low-freq triangle wave
	duty = LFTri.kr(3).range(0.4,0.6);

	//low-freq pulse wave
	freq = LFPulse.kr(4,0,0.1).range(300,600);

	//our audio rate pulse wave, controlled
	//by control-rate UGens
	sig = Pulse.ar(freq, duty, amp);
};
)

x = f.play;
x.free;



//Let's continue with this example in order to demonstrate other methods and features, but feel free to modify the values and provide arguments of your own.

(
//SCOPE
//The 'scope' method will play a function,
//and also open a GUI window displaying the
//waveform of the output audio.
f = {
	var sig, freq, duty, amp;
	amp = LFNoise1.kr(20).exprange(0.02,0.5);
	duty = LFTri.kr(3).range(0.4,0.6);
	freq = LFPulse.kr(4,0,0.1).range(300,600);
	sig = Pulse.ar(freq, duty, amp);
}.scope;
)

(
//PLOT
//Functions can be plotted. This example uses plot's default values.
f = {
	var sig, freq, duty, amp;
	amp = LFNoise1.kr(20).exprange(0.02,0.5);
	duty = LFTri.kr(3).range(0.4,0.6);
	freq = LFPulse.kr(4,0,0.1).range(300,600);
	sig = Pulse.ar(freq, duty, amp);
}.plot;
)

(
//Here we specify our own duration argument (0.5 instead of the
//default 0.01). The number of seconds specified must elapse
//before Plotter can display the requested waveform. See the plot
//method within the Function help file for more information.
f = {
	var sig, freq, duty, amp;
	amp = LFNoise1.kr(20).exprange(0.02,0.5);
	duty = LFTri.kr(3).range(0.4,0.6);
	freq = LFPulse.kr(4,0,0.1).range(300,600);
	sig = Pulse.ar(freq, duty, amp);
}.plot(0.5);
)



//MULTICHANNEL EXPANSION

//In the context of UGens, Arrays translate to multichannel sound. Previous examples have been monophonic and produce sound in the lowest-numbered hardware output channel (i.e. the left speaker, most likely).

//For the sake of variety, some values have been arbitrarily changed.
(
f = {
	var sig, freq, duty, amp;
	amp = LFNoise1.kr(20).exprange(0.01,0.3);
	duty = LFTri.kr(0.3).range(0.01,0.99);

	//our freq varable is now a 2-channel LFPulse
	//UGen. The left channel has a frequency of
	//4Hz, the right, 5Hz.
	freq = LFPulse.kr([4,5],0,0.1).range(400,500);

	//As a result, the UGen that LFPulse controls
	//is ALSO converted to 2-channel sound.
	sig = Pulse.ar(freq, duty, amp);
};
)

x = f.play;
x.free;

//In this example, even more values have been converted to Arrays of size 2. In particular, the amplitude control now differs between the 2 channels, so the sound "moves" quite a bit more.
(
f = {
	var sig, freq, duty, amp;

	//an LFNoise1 for the left channel, and
	//a different LFNoise1 for the right channel.
	amp = LFNoise1.kr([8,8.1]).exprange(0.01,0.3);

	duty = LFTri.kr(3).range(0.01,0.5);
	freq = LFPulse.kr([4,5],0,0.1).range(150,200);
	sig = Pulse.ar(freq, duty, amp);
};
)

x = f.play;
x.free;




//RETURNING THE LAST LINE

//Recall that functions will return the last statement of code. Therefore, it is quite easy to silence your function by appending a statement of "0;" to your function definition. Likewise, it's easy to isolate a particular variable by appending a line with only that variable's name.
(
x = {
	var sig, freq, duty, amp;
	amp = LFNoise1.kr([8,8.1]).exprange(0.01,0.3);
	duty = LFTri.kr(3).range(0.01,0.5);
	freq = LFPulse.kr([4,5],0,0.1).range(150,200);
	sig = Pulse.ar(freq, duty, amp);

	//Silent! All our code is for naught.
	//However, our UGens still use CPU, cycles,
	//even if we output silence.
	0;
}.play;
)

x.free; //free the silent Synth, even though we
//dont' hear anything while its active.


//In this example, we only hear sig2, since it is
//the last statement:
(
x = {
	var sig, sig2, freq, duty, amp;
	amp = LFNoise1.kr([8,8.1]).exprange(0.01,0.3);
	duty = LFTri.kr(3).range(0.01,0.5);
	freq = LFPulse.kr([4,5],0,0.1).range(150,200);
	sig = Pulse.ar(freq, duty, amp);
	sig2 = SinOsc.ar([400,300],mul:0.3);
}.play;
)

x.free;

//But here, the last line is now the sum of sig and sig2:
(
x = {
	var sig, sig2, freq, duty, amp;
	amp = LFNoise1.kr([8,8.1]).exprange(0.01,0.3);
	duty = LFTri.kr(3).range(0.01,0.5);
	freq = LFPulse.kr([4,5],0,0.1).range(150,200);
	sig = Pulse.ar(freq, duty, amp);
	sig2 = SinOsc.ar([400,300],mul:0.3);
	sig + sig2
}.play;
)

x.free;

//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////


// ENVELOPES

//In a pinch, you can use Line and XLine as envelope controls:
(
x = {
	var sig, env;
	sig = PinkNoise.ar;

	//this UGen travels from 1 to 0 over 4 seconds
	env = Line.kr(1,0,4, doneAction:2);

	sig = sig * env;
}.play;
)

//If you do not specify doneAction:2, your Synth will continue to generate values (and therefore consume CPU cycles) even after the envelope has completed. Search the help documentation for "UGen done-actions" for more information, although I have rarely found the need to use a doneAction other than doneAction:2.

//EnvGen is a much more flexible envelope generator:

(
f = {
	var sig, env;
	sig = PinkNoise.ar;

	//a control rate EnvGen
	env = EnvGen.kr(

		//the first argument is the type of
		//envelope. Here we're using a
		//percussive envelope with its default
		//values. See the Env and EnvGen help
		//files for more information.
		Env.perc,
		doneAction:2
	);

	sig = sig * env;
};
)

x = f.play; //No need to free this Synth, since doneAction:2 handles this.

(
f = {
	arg trig=0;
	var sig, env;
	sig = PinkNoise.ar;
	env = EnvGen.kr(

		//here's an adsr envelope:
		//attack, decay, sustainLevel, release
		Env.adsr(0.01,0.5,0.3,4),

		//and a gate argument
		trig,
		doneAction:2
	);

	sig = sig * env;
};
)

//Our trigger argument is zero by default,
//so the adsr envelope has not begun its
//attack portion.
x = f.play;

//set trig to 1 to open the gate and hold
//at the sustain level:
x.set(\trig, 1);

//and set trig to 0 to trigger the release.
x.set(\trig, 0);

//Note that all arguments can be "set" in this fashion, not just triggers and gates:
x = {arg freq=440; SinOsc.ar(freq, mul:0.2)}.play;

x.set(\freq, 200);
x.set(\freq, 2000);

//A random value between 40 and 2000, with exponential distribution. Run this statement multiple times. Also, search for the help document titled "Randomness" for more information.
x.set(\freq, exprand(40,2000));

//free the Synth when done.
x.free;


//Lastly, Env.new is one of the most open-ended and flexbile type of envelope:
(
f = {
	var sig, env;
	sig = PinkNoise.ar;
	env = EnvGen.kr(
		Env.new(
			//Env.new expects an array of levels,
			[0,1,0.5,0],

			//an array of times (seconds),
			[2,3,1],

			//and an array of curves.
			//For the curve array, 0 is linear,
			//positive values will interpolate
			//slowly, then quickly, while negative
			//values will do the opposite.
			[1,0,-1]

		),
		doneAction:2
	);

	sig = sig * env;
};
)

x = f.play;

//"plot" is also defined for the Env class. Experiment with the following values:
Env.new([0,1,0.5,0],[2,3,1],[1,0,-1]).plot;



//Dust and Impulse generate triggers. Generally speaking, a trigger is a signal that rests at a non-positive value, momentarily rises to a positive value, and immediately returns to rest position. Specifically, The former generates non-periodic triggers at a given approximate density, and the latter generates periodic triggers and a given frequency. They can be used as audio generators by themselves:

x = {Dust.ar(8)}.play; //approximately 8 triggers per second.
x = {Impulse.ar(8)}.play; //exactly 8 triggers per second.
x = {Impulse.ar(MouseX.kr(1,150,1).poll)}.play; //move the mouse along the horizontal axis: values beyond 20Hz produce a perceivable tone

//Dust and Impulse can be useful for autonomously triggering envelopes:
(
x = {
	arg trig=0;
	var sig, env;
	sig = PinkNoise.ar;
	env = EnvGen.kr(

		//percussive envelope
		Env.perc(0.005,0.1),

		//Dust will trigger the envelope approximately 4 times per second
		Dust.kr(4)

		//note the absence of doneAction:2, which
		//allows the envelope to be re-triggered many times
	);

	sig = sig * env;
}.play;
)




//SoundIn.ar accesses your microphone or other audio hardware inputs.
//WARNING: USE HEADPHONES OR YOU MAY EXPERIENCE SCREECHING FEEDBACK!!!
(
//here, we simply output the input signal:
x = {
	var sig;
	sig = SoundIn.ar(0); //0 indicates we are using the lowest numbered input on your hardware device
}.play;
)

