/* BUFFERS */
//It is very common to play and process recorded audio files. In SuperCollider, audio files are loaded into memory space called Buffers. See the Buffer help file for detailed information.

s = Server.local.boot;

//read a sound file into a buffer from an absolute path. You'll have to modify the path name so that you can access a sound file on your own computer.
~glass = Buffer.read(s, "/Users/eli/Sounds/scaudio/glass_break_03.aiff");

//as a convenience, you can use the method 'play' to hear the contents of a buffer
~glass.play;

//Instances of buffer respond to certain methods:
~glass.numChannels; //number of channels
~glass.duration; //duration
~glass.numFrames; //number of frames (number of samples = number of frames x number of channels)
~glass.bufnum; //every allocated buffer has a unique integer "bufnum" associated with it


//UGens that interact with Buffers:

//PlayBuf (see help file for more information)
(
f = {
	arg rate=1, loop=0;
	var sig;
	sig = PlayBuf.ar(
		1, //number of channels (non-modulatable)
		~glass.bufnum, //buffer number (bufnum)
		BufRateScale.kr(~glass) * rate, //playback rate (BufRateScale ensures buffer sample rate matches the audio server sample rate)
		loop:loop, //determines whether the buffer will loop (0 or 1)
		doneAction:2 //doneAction:2 frees the Synth when the buffer has finished
	);
};
)

f.play(args:[\rate, 1]);

f.play(args:[\rate, 0.7]); //slower playback rate
f.play(args:[\rate, 0.3]);

x = f.play(args:[\rate, 2, \loop, 1]); //faster playback rate, with looping

x.set(\loop, 0); //turn off looping


//BufRd (see help file for more information)
//Unlike Playbuf, BufRd uses an audio rate index into the buffer.
(
{
	var sig;
	sig = BufRd.ar(
		1,
		~glass.bufnum,

		//Line as an .ar index
		Line.ar(
			0, //start at frame zero
			BufFrames.kr(~glass.bufnum), //end at the last frame of the buffer
			BufDur.kr(~glass.bufnum), //line duration is equal to buffer duration
			doneAction:2
		)
	)
}.play;
)

(
{
	var sig;
	sig = BufRd.ar(
		1,
		~glass.bufnum,

		//using a sine wave as an index into the buffer
		SinOsc.ar(0.2,3pi/2).range(0,BufFrames.kr(~glass.bufnum))
	)
}.play;
)


(
f = {
	arg rate=1;
	var sig;
	sig = BufRd.ar(
		1,
		~glass.bufnum,

		//phasor is a periodic linear ramp oscillator, ideal for looping a sound file with BufRd.
		Phasor.ar(
		rate:rate,
		end:BufFrames.kr(~glass.bufnum)
		)
	)
};
)

x = f.play;
x.set(\rate, 2);   //twice as fast
x.set(\rate, 0.5); //twice as slow





//RecordBuf

//allocate a four-second buffer for recording
~myBuf = Buffer.alloc(s,s.sampleRate*4,1);

~myBuf.play; //playing this buffer now produces silence (it's full of zeros)

(
//evaluate and speak into the microphone for approximately four seconds
{
	var input;

	input = SoundIn.ar(0);

	RecordBuf.ar(
		input,
		~myBuf,
		doneAction:2,
		loop:0
	);

	0; //end the function with 0 if you don't want to hear the audio being recorded
}.play;
)

~myBuf.play; //we've filled the buffer with 4 seconds of microphone input

//play it backwards
x = {PlayBuf.ar(1, ~myBuf.bufnum, -1, 1, ~myBuf.numFrames-2, doneAction:2)}.play;

~myBuf.zero; //buffers can be zeroed at any time

~myBuf.play; //recorded audio is gone

~myBuf.free; //freeing a buffer removes it from the server and allows the bufnum to be re-used for other allocated buffers

Buffer.freeAll; //free all buffers



//allocate a 1 second buffer
~myBuf = Buffer.alloc(s,s.sampleRate,1);

//record/playback simultaneously with RecordBuf and PlayBuf (feedback delay-like effect)
(
{
	var input, rec, play;

	input = SoundIn.ar(0);
	rec = RecordBuf.ar(
		input,
		~myBuf,
		recLevel:1,
		preLevel:-3.dbamp, //buffer contents are not entirely overwritten, but instead are reduced by 3dB and mixed with the incoming signal
		loop:1
	);
	play = PlayBuf.ar(
		1!2,
		~myBuf,
		loop:1
	);

}.play;
)



