/* INTRODUCTION TO MIDI */

//For this lecture, I recommend having a MIDI keyboard controller powered and connected to your computer before opening SuperCollider.

//Detect and connect with all MIDI devices (may take a few seconds)
MIDIIn.connectAll;

//List available MIDI devices
MIDIClient.sources;




//MIDIdef is an object that responds to MIDI messages by evaluating a function. The values of the MIDI message can be passed into the function as input arguments. See the help file for MIDIdef for more information. The help file called "Using MIDI" is also a good reference guide.

//Evaluate the following code to instantiate four MIDIdefs. These objects will respond to noteOns, noteOffs, continuous controllers, and bend messages, respectively. Their functions simply post the values received.
(
//receive note ons
MIDIdef.noteOn(\printNoteOn, {
	arg val, num, chan, src;
	("note on:"++[val,num,chan,src]).postln;
});

//receive note offs
MIDIdef.noteOff(\printNoteOff, {
	arg val, num, chan, src;
	("note off:"++[val,num,chan,src]).postln;
});

//receive CC messages
MIDIdef.cc(\printCC, {
	arg val, num, chan, src;
	("cc msg:"++[val,num,chan,src]).postln;
});

//pitch bend
MIDIdef.bend(\printBend, {
	arg val, chan, src;
	("bend:"++[val,chan,src]).postln;
});
)




//If you instantiate a new MIDIdef with a symbolic key that is already in use (in this case, \printNoteOn), SC will replace the old MIDIdef with the new one, rather than create a second MIDIdef.
(
MIDIdef.noteOn(\printNoteOn, {
	arg val, num, chan, src;
	"hi".postln;
});
)

//noteOns now print "hi" INSTEAD of an array of values.

//MIDIdefs can be disabled an enabled
MIDIdef(\printNoteOn).disable;
MIDIdef(\printNoteOn).enable;

//Remove all MIDIdefs
MIDIdef.freeAll;

//command-period will also free all MIDIdefs.




//Making sound using MIDIdefs
//simply play a note with a percussive envelope (and doneAction:2, therefore we don't need to worry about noteOff messages just yet)

(
//Two MIDIdefs, one plays a sine tone, the other plays pink noise. They can be independently enabled and disabled.
MIDIdef.noteOn(\playNote, {
	arg val, num, chan, src;
	{
		var sig, env;
		sig = SinOsc.ar(num.midicps);
		env = EnvGen.kr(Env.perc, doneAction:2);
		sig = sig * env;
		sig = sig * val.linexp(1,127,0.05,0.8);
	}.play;
});

MIDIdef.noteOn(\playNoise, {
	arg val, num, chan, src;
	{
		var sig, env;
		sig = PinkNoise.ar;
		env = EnvGen.kr(Env.perc, doneAction:2);
		sig = sig * env;
		sig = sig * val.linexp(1,127,0.05,0.8);
	}.play;
});
)

//disable noise
MIDIdef.noteOn(\playNoise).disable;

//enable noise, disable sine tone
(
MIDIdef.noteOn(\playNote).disable;
MIDIdef.noteOn(\playNoise).enable;
)

MIDIdef.freeAll;







//note on & note off - how NOT to do it

//if a second noteOn is received before a noteOff, then x is overwritten with a new Synth, and the Synth that was played first cannot be accessed and therefore cannot be freed. This is a delightful recipe for "stuck notes".
(
MIDIdef.noteOn(\on, {
	arg val, num, chan, src;
	x = {
		arg trig=1;
		var sig, env;
		sig = SinOsc.ar(num.midicps);
		env = EnvGen.kr(Env.adsr,trig,doneAction:2);
		sig = sig * env;
		sig = sig * val.linexp(1,127,0.05,0.8);
		sig!2
	}.play;
});

MIDIdef.noteOff(\off, {x.set(\trig,0)});
)


//note on & note off - improved, but not best
(
//create an array of ample size
~noteArray = Array.newClear(128);

MIDIdef.noteOn(\on, {
	arg val, num, chan, src;

	//Synths will be placed into the array
	//at the index specified by their note number
	~noteArray[num] = {
		arg trig=1;
		var sig, env;
		sig = SinOsc.ar(num.midicps);
		env = EnvGen.kr(Env.adsr,trig,doneAction:2);
		sig = sig * env;
		sig = sig * val.linexp(1,127,0.05,0.8);
		sig!2
	}.play;
});

MIDIdef.noteOff(\off,
	{
		arg val, num, chan, src;

		//note offs trigger a fade out on
		//the synth at the index specified by
		//their note number
		~noteArray[num].set(\trig,0);
	}
);
)

MIDIdef.freeAll;






//The example above is alright, but it entangles audio processing with language-side MIDI data and synth triggering - it's not as efficient as it could be. A better option is to define a sound function and create an array of synths FIRST, and simply trigger/release them as noteOns/noteOffs are received. In other words, don't make the MIDIdef responsibe for instantiating Synths, rather, make it responsible for triggering Synths that already exist.

(
//create a sound function
f = {
	arg notnum=60, vel=64, trig=0;
	var sig, env;
	sig = SinOsc.ar(notnum.midicps);
	env = EnvGen.kr(Env.adsr,trig,doneAction:0);
	sig = sig * env;
	sig = sig * vel.linexp(1,127,0.05,0.8);
	sig!2
};

//create an Array of Synths. Recall that if the curly braces are omitted, SC will fill the array with 128 copies of a singular instance of Synth, whereas enclosing f.play in curly braces means that SC will re-evaluate the function as each index is filled, so each Synth will be unique and independently addressable.
~noteArray = Array.fill(128,{f.play});

MIDIdef.noteOn(\on, {
	arg val, num, chan, src;

	~noteArray[num].set(\notnum, num, \vel, val, \trig, 1);
});

MIDIdef.noteOff(\off,
	{
		arg val, num, chan, src;
		~noteArray[num].set(\trig,0);
	}
);
)

MIDIdef.freeAll;
