/* SERVER ARCHITECTURE */

//INTRODUCTION

//Using {function}.play is quick and convenient, but it is not ideal for taking advantage of the subtleties of scsynth's architecture. The most notable classes that deal with scsynth's content and structure are Node, Group, Synth, SynthDef and Bus.

//Here's a quick introduction to the first three classes, Node, Group, and Synth:

//Node is an abstract superclass, which means the user generally has no need to create a Node explicitly (i.e. you would never do something like x = Node.new). However, Node exists in order to pave the way for its two subclasses, Group and Synth. A Synth is a collection of interconnected UGens that produce a signal, and we've already been making Synths via {function}.play. A Group, on the other hand, is a collection of ordered Nodes (the Nodes inside of a Group can be Synths or Groups).

//The placement of Nodes on the server is very important, since Nodes are processed from first to last once per control cycle. In SC terms, this is referred to as "head to tail." Therefore, if Synth B takes its signal input from the output of Synth A, then Synth A must be placed before Synth B (i.e. Synth A must be closer to the head of the Node chain than Synth B).

//The following help file is a great starting point for understanding how the SC server operates. Certainly read the first two sections, "Introduction" and "Main Design Concepts," but don't worry too much about the rest.

Server Architecture



//SYNTHDEF & SYNTH

//When you play a function, SC secretly creates a SynthDef file and exectues the specified sound by instantiating a Synth. This is why evaluating {function}.play posts something along the lines of "Synth("temp__0" : 1000)" in the post window. It's actually quite trivial to convert a UGen function to a SynthDef; there are only three simple rules to remember:


/*

1. A SynthDef needs, at minimum, two arguments: a name (specified as a String or Symbol, much like MIDIdef), and a UGen function.

2. If you want your Synth to output a signal, you must include at least one Out UGen in your UGen function. When you use {function}.play, SC creates an Out.ar UGen for you. See the Out help file for more information.

3. Use the ".add" method on your newly created SynthDef, or else it will not be added to the SynthDef library. There are other methods which allow access to newly created SynthDef files, but ".add" is the most convenient and all-purpose. See the SynthDef help file for more information and more methods.

*/

//We'll be making sound, so first we'll configure the server and then boot it.
s.options.numOutputBusChannels = 8;
s.options.numInputBusChannels = 8;
s.reboot;

//Let's convert a UGen function to a SynthDef. Here's the original function from my Assignment01:
(
x = {
	arg gate=1, fund=50;
	var sig, bpfHz, amp, env;

	env = EnvGen.kr(
		Env.adsr(10,0.1,1,10),
		gate,
		doneAction:2);

	amp = LFNoise1.kr(
		{exprand(0.1,0.8)}!12).exprange(0.03,1);

	bpfHz = LFNoise0.kr(
		{[0.05,0.1,0.2,0.25,1/3].choose}!12
	).exprange(fund*2,fund*15);
	bpfHz = bpfHz.round(fund);

	sig = PinkNoise.ar(1!12);
	sig = BPF.ar(sig, bpfHz, 400.reciprocal) * amp;
	sig = Splay.ar(sig);
	sig = BLowShelf.ar(sig, 200, 0.5, 6) * 20 * env;
}.play;
)

x.free

//Here's the conversion to SynthDef:
(
SynthDef.new(\ethereal, {
	arg gate=1, fund=50, out=0;
	var sig, bpfHz, amp, env;

	env = EnvGen.kr(
		Env.adsr(10,0.1,1,10),
		gate,
		doneAction:2);

	amp = LFNoise1.kr(
		{exprand(0.1,0.8)}!12).exprange(0.03,1);

	bpfHz = LFNoise0.kr(
		{[0.05,0.1,0.2,0.25,1/3].choose}!12
	).exprange(fund*2,fund*15);
	bpfHz = bpfHz.round(fund);

	sig = PinkNoise.ar(1!12);
	sig = BPF.ar(sig, bpfHz, 400.reciprocal) * amp;
	sig = Splay.ar(sig);
	sig = BLowShelf.ar(sig, 200, 0.5, 6) * 20 * env;
	Out.ar(out, sig);
}).add;
)

//Notice that I've supplied a user-chosen name (\ethereal), I've included an Out.ar at the end of my function, and I've added this new SynthDef using the ".add" method. I've also included a new argument (outBus) to specify the audio bus to which to the output signal will be sent. Other than this, nothing has changed.

//We can play the sound specified in this SynthDef using the Synth object. A Synth needs, at minimum, one argument: the name of a SynthDef.

x = Synth.new(\ethereal);

x.free;

//You can provide an array of arguments as an optional second argument to Synth, very much like how we've been providing arguments with {function}.play:

x = Synth.new(\ethereal, [\fund, 40]);

//Don't forget about the "set" command:

x.set(\fund, 60);

x.free;


//OUT.AR & HARDWARE OUTPUT BUSSES

//An audio output bus index of 0 (which I've provided as the default value for our 'outBus' argument in our \ethereal SynthDef) corresponds to the lowest-numbered output channel on the current audio hardware device being used by scsynth. The SC audio bus with index 1 corresponds to the next-lowest hardware output, and so on.

//If you are outputting a multichannel signal, SC will write the first channel to the bus provided in Out.ar's first argument, and will write the subsequent channels to numerically ascending consecutive busses.

//Our ethereal SynthDef plays a stereo signal, and the output bus is set to 0. Therefore, we hear the first channel on bus 0 (usually the left speaker) and the second channel on bus 1 (right speaker). If we were to specify outBus = 2, then these stereo channels would play on busses 2 and 3, respectively. You won't hear this on a laptop, but view the level meters with s.meter; or press command+m to see for yourself:

x = Synth.new(\ethereal, [\out, 2]);

s.meter;

x.free;

//If you want to output consecutive audio channels on non-consecutive busses, you will have to make more than one output UGen (however I essentially never have the need to do so):

(
SynthDef.new(\etherealMultiOut, {
	arg gate=1, fund=50, out1=0, out2=1;
	var sig, bpfHz, amp, env;

	env = EnvGen.kr(
		Env.adsr(10,0.1,1,10),
		gate,
		doneAction:2);

	amp = LFNoise1.kr(
		{exprand(0.1,0.8)}!12).exprange(0.03,1);

	bpfHz = LFNoise0.kr(
		{[0.05,0.1,0.2,0.25,1/3].choose}!12
	).exprange(fund*2,fund*15);
	bpfHz = bpfHz.round(fund);

	sig = PinkNoise.ar(1!12);
	sig = BPF.ar(sig, bpfHz, 400.reciprocal) * amp;
	sig = Splay.ar(sig);
	sig = BLowShelf.ar(sig, 200, 0.5, 6) * 20 * env;

	//output the first channel in the multichannel signal
	//on outBus1, and output the other channel on outBus2.
	Out.ar(out1, sig[0]);
	Out.ar(out2, sig[1]);
}).add;
)

//Again, view the level meters to see for yourself.
~nonConsec = Synth.new(\etherealMultiOut, [\out1, 0, \out2, 2]);

s.meter;


//Out responds to multichannel expansion with Arrays just like any other UGen. If your signal output is monophonic, you can multichannel expand an output UGen to duplicate the output on other busses:
(
SynthDef.new(\etherealMono, {
	arg gate=1, fund=50;
	var sig, bpfHz, amp, env;

	env = EnvGen.kr(
		Env.adsr(10,0.1,1,10),
		gate,
		doneAction:2);

	amp = LFNoise1.kr(
		{exprand(0.1,0.8)}!12).exprange(0.03,1);

	bpfHz = LFNoise0.kr(
		{[0.05,0.1,0.2,0.25,1/3].choose}!12
	).exprange(fund*2,fund*15);
	bpfHz = bpfHz.round(fund);

	sig = PinkNoise.ar(1!12);
	sig = BPF.ar(sig, bpfHz, 400.reciprocal) * amp;
	sig = Splay.ar(sig);

	//mix down to a single channel
	sig = Mix(sig);
	sig = BLowShelf.ar(sig, 200, 0.5, 6) * 20 * env;

	//play this monophonic signal on busses 1 and 2.
	//listening on headphones will make the monophonic
	//effect very audible.
	Out.ar([0,1], sig);
}).add;
)

x = Synth.new(\etherealMono);
s.meter;
x.free;

//However, multichannel expansion on an output UGen WHILE ALSO outputting a multichannel signal will almost certainly not have the desired effect:
(
SynthDef.new(\etherealBadMultiOut, {
	arg gate=1, fund=50;
	var sig, bpfHz, amp, env;

	env = EnvGen.kr(
		Env.adsr(10,0.1,1,10),
		gate,
		doneAction:2);

	amp = LFNoise1.kr(
		{exprand(0.1,0.8)}!12).exprange(0.03,1);

	bpfHz = LFNoise0.kr(
		{[0.05,0.1,0.2,0.25,1/3].choose}!12
	).exprange(fund*2,fund*15);
	bpfHz = bpfHz.round(fund);

	sig = PinkNoise.ar(1!12);
	sig = BPF.ar(sig, bpfHz, 400.reciprocal) * amp;
	sig = Splay.ar(sig);

	//don't mix stereo to mono
	//sig = Mix(sig);

	sig = BLowShelf.ar(sig, 200, 0.5, 6) * 20 * env;

	//output the stereo signal on multiple busses
	Out.ar([0,1], sig);
}).add;
)

//Because SC distributes multiple channels in ascending numerical order, the stereo sound will play on busses 0 & 1 AND on busses 1 & 2. Hence bus 1 will contain the left and right channel.

x = Synth.new(\etherealBadMultiOut);
s.meter; //<-- see the level meters for visual proof.
x.free;

//The lesson here: either create a multichannel signal on a single bus, OR, multichannel-expand an output UGen with a monophonic signal. Not both!




//INS AND OUTS: SYNTHS THAT SHARE SIGNAL DATA

//Here's a SynthDef that generates a sinusoid burst every two seconds and applies a 3-second reverb effect:
(
SynthDef(\blipsAndReverb, {
	arg freq=880, amp=0.5, out=0;
	var imp, env, sig;
	imp = Impulse.kr(0.5);
	env = EnvGen.kr(Env.perc(0.01,0.2),imp);
	sig = SinOsc.ar(freq);
	sig = sig * env * amp;
	sig = GVerb.ar(sig, 99, 3);
	Out.ar(out, sig);
}).add;
)

x = Synth.new(\blipsAndReverb);

//When we free the Synth, both the sine bursts AND reverb grind to a halt:
x.free;

//It sure would be nice if we could stop the generated sound without stopping the reverb effect. We can accomplish this by creating two SynthDefs instead of one.

//Notice that a few new arguments have been added, since the Synths will now be sharing signal data via busses. By default, the sine tones will be written to audio bus 50, and the reverb will read from bus 50 while outputting its signal to bus 0 (hardware output).
(
SynthDef(\blips, {
	arg freq=880, amp=0.5, out=50;
	var imp, env, sig;
	imp = Impulse.kr(0.5);
	env = EnvGen.kr(Env.perc(0.01,0.2),imp);
	sig = SinOsc.ar(freq);
	sig = sig * env * amp;
	Out.ar(out, sig);
}).add;

SynthDef(\reverb, {
	arg in=50, out=0;
	var sig;
	sig = In.ar(in, 1);
	sig = GVerb.ar(sig, 99, 3);
	Out.ar(out, sig);
}).add;
)

//Execute these two Synths, one at a time, in this order:
~verb = Synth(\reverb);
~blips = Synth(\blips);

//Now we can free the generated sound without interfering with the reverb effect:
~blips.free;

//Re-instantiate the blips, and the reverb effect remains present (assuming you haven't pressed command-period)
~blips = Synth(\blips);

//free it again
~blips.free;

//And we can free the reverb Synth at leisure:
~verb.free;

//Suppose we casually and innocently evaluated these Synths in the opposite order:
~blips = Synth(\blips);
~verb = Synth(\reverb);

//We hear nothing! Sure enough, If we bring up the visual node tree:
s.plotTree;

//we can see that our blips and reverb are both happily existing on the server.  But why no sound!?

s.freeAll;

//This brings us to the importance of Order of Execution. As mentioned in the introduction to this document, Nodes are processed from head to tail. By default, a new Node (Synth or Group) is added to the head of the default group on the default server.

//Consider this example again (but don't re-evaluate the code):
~blips = Synth(\blips);
~verb = Synth(\reverb);

//Here, we first instantiate a \blips Synth called ~blips. It gets added to the head of the default group - no problem there. Then, we add our reverb Synth, which depends on ~blips' output, and so it must be placed AFTER ~blips in order for us to hear the output. But, since we do not specify otherwise, SC adds ~verb to the head of the default group, which positions it BEFORE ~blips. No good!

//Fortunately, Synth accepts two more arguments: a target and an addAction. See the Synth help file for more information. We can instantiate our nodes in any order we like, so long as we provide proper targets and addActions. There are usually several combinations of target and addAction that will achieve the desired results, but here's one:

//do this first:
~blips = Synth(\blips);

//add ~verb immediately after ~blips
~verb = Synth(\reverb, target:~blips, addAction:\addAfter);

s.freeAll;


//This would also work:
~blips = Synth(\blips);

//add ~verb to the tail of the default group on Server.local
~verb = Synth(\reverb, target:Server.local, addAction:\addToTail);

//let's free all the Nodes once more and discuss Groups:
Server.local.freeAll;





//GROUPS

//Groups are convenient containers for concepually similar Synths. You might, for example, have a generator group near the head of your Node chain, and various effects Groups near the tail of your Node chain. Once you've instantiated your Groups, you only need to dump your Synths into the proper Groups without worrying about addActions. Because the order of the Groups has already been established, the Synths contained within these Groups will preserve the Group-level Node order.

//Bear in mind that Groups and Synths share many methods. Groups can be "set" and "freed" just like Synths. When a Group recieves a message, it passes that message to all the Nodes it contains. Therefore, if a Group is freed, All Nodes within will receive the "free" message (the Group itself will also be freed).

//Here's a simple example of this concept. I recommend having the visual Node tree visable while executing the following code:

//create a group for generative Synths
~genGroup = Group.new;

//".after" is a convenient method for creating a Group after a specific Node. See the help file on Group for more information and more methods.
~fxGroup = Group.after(~genGroup);

s.plotTree; //<--view our two Groups

//Now, we only need to specify the proper Group as targets for our Synths. I'm getting really sick of 880Hz, so I'm specifying a different values for the frequency argument:
~blips = Synth(\blips, [\freq,300], ~genGroup);
~verb = Synth(\reverb, target:~fxGroup);

//Free the Nodes:
~blips.free;
~verb.free;

//Here's an alternative way to generate your two Groups, this time taking advantage of the default target and addAction for new nodes. Remember that every new Node (Group or Synth), unless specified otherwise, will be added to the head of the default group on the default server.

~fxGroup = Group.new;
~genGroup = Group.new;

~blips = Synth(\blips, target:~genGroup);
~verb = Synth(\reverb, target:~fxGroup);

s.plotTree;

~blips.free;

Server.local.freeAll; //free all Synths AND Groups



//In class, we briefly mentioned the possible confusion resulting from seeing these two groups nested inside the default group. In other words, why do we need the default group at all? Can't we just make two Groups on the server?

//The answer is no. We need this default Group as a starting point for building our Node tree. Without a default group, it is impossible (or at least, very annoying) to make new Nodes. If you really wanted to, you can free the default group like so:

s.sendMsg('/n_free', 1);

//If you just freed the default group, I'm sorry to say that you'll have to zero your hard drive and reinstall your operating system, possibly even buy a new computer. Just kidding. You can press [command]+[period] and scsynth will revert to its initial state, default group and all.

//And, If you did free the default group, notice that we can no longer create new Nodes in the usual way:

~genGroup = Group.new; //note the error message

s.plotTree; //sad, empty, nothingness

//To return to normal, just press command-period or evaluate the following line.

s.freeAll;




//BUSSES

//In our previous example, we used 50 as an audio bus index on which Synths would read and write data internally. This works just fine, but it's not the safest solution. To elaborate, we must first discuss SC busses in a bit more depth.

//On startup, SC allocates a number of audio busses and control busses. You can see how many of each you have available by evaluating the following two lines (these attributes are accessed via the ServerOptions class, which we have already discussed).

s.options.numAudioBusChannels;
s.options.numControlBusChannels;

//SC associates its lowest numbered audio busses with your hardware's outputs and inputs, respectively. SC assumes a certain number of output busses and input busses upon startup, which you can view using the following two lines.

s.options.numOutputBusChannels;
s.options.numInputBusChannels;

//If the previous two value are both 8, This means the SC audio bus with indices 0-7 correspond to your hardware outputs, and 8-15 correspond to your hardware inputs.

//If you are on a laptop, you might consider evaluating these two lines, and then rebooting the server:

s.options.numOutputBusChannels=2;
s.options.numInputBusChannels=1;
s.reboot;

//Once the server has rebooted, run these lines again, and you can see that the number of hardware busses has been updated:

s.options.numOutputBusChannels;
s.options.numInputBusChannels;

//Also, bring up the level meters with [command]+[m] or s.meter; and you'll see that they've also been updated to reflect the change in your server's options.

s.meter;

//Let's return to our discussion of using 50 (or any integer for that matter) as an internal bus index. The reason why this is not the best idea should now be evident: the number of hardware output/input busses is flexible. Generally speaking, 50 is probably high enough to be safe, but there's no guarantee. Therefore, it is highly preferable to use the Bus class to allocate available internal busses:

//SC will return the lowest available audio bus, which the client can access via the global variable "b":
b = Bus.audio(s,1);

//The second argument tells SC how many channels you want. Like output UGens, SC will allocate additional busses consecutively, in ascending numerical order. Let's say we wanted a four-channel bus:
c = Bus.audio(s,4);

//and a two-channel bus:
d = Bus.audio(s,2);

//Look at the information in the post window. It might look something like this:

/*
Bus(audio, 3, 1, localhost)
Bus(audio, 4, 4, localhost)
Bus(audio, 8, 2, localhost)
*/

//First, SC allocated the lowest available bus (index 3). Then, it allocated the next four busses (4 through 7). The next available bus index is 8, so busses 8 and 9 are allocated for the stereo bus.

//Using our new knowledge about SC busses, let's modify our previous SynthDefs and Synths.

//It's actually not necessary to re-"add" these SynthDefs unless you've quit and re-opened SC. The're just here so you don't need to scroll up and down like a madman.
(
SynthDef(\blips, {
	arg freq=880, amp=0.5, out=0;
	var imp, env, sig;
	imp = Impulse.kr(0.5);
	env = EnvGen.kr(Env.perc(0.01,0.2),imp);
	sig = SinOsc.ar(freq);
	sig = sig * env * amp;
	Poll.kr(Impulse.kr(50), sig);
	Out.ar(out, sig);
}).add;

SynthDef(\reverb, {
	arg in=50, out=0;
	var sig;
	sig = In.ar(in, 1);
	sig = GVerb.ar(sig, 99, 3);
	Out.ar(out, sig);
}).add;
)

//instantiate Groups:
~genGroup = Group.new;
~fxGroup = Group.after(~genGroup);

//supply our allocated bus as an argument for these Synths:
~blips = Synth(\blips, [\out, b], ~genGroup);
~verb = Synth(\reverb, [\in, b, \out, 0], ~fxGroup);

~blips.free;

Server.local.freeAll