//The follow code provides us with a basic modular synthesizer (without GUI). This code does not incorporate ServerTree, so clearing the server with [command]+[period] will free all the Groups, which will need to be reinstantiated to run the code a second time. See the ServerTree version of this code for improvements.

s.boot;

(
SynthDef(\mod, {
	arg freq=8, modType=0, outbus;
	var sig;
	sig = Select.kr(
		which:modType,
		array:[
			//all mod controls range from 0 to 1
			LFDNoise0.kr(freq).range(0,1),
			LFDNoise1.kr(freq).range(0,1),
			SinOsc.kr(freq).range(0,1),
			LFSaw.kr(freq).range(0,1),
			LFTri.kr(freq).range(0,1)
		]
	);
	Out.kr(outbus, sig);
}).add;

SynthDef(\trig, {
	arg inbus, freqScale=1, baseFreq=0, trigType=0, len=0.2, outbus;
	var in, sig;
	in = In.kr(inbus) * freqScale + baseFreq;
	sig = Select.kr(
		which:trigType,
		array:[
			Impulse.kr(baseFreq),
			Impulse.kr(in),
			Dust.kr(in)
		]
	);
	sig = Trig.kr(sig, len);
	Out.kr(outbus, sig);
}).add;

SynthDef(\adsr, {
	arg	inbus, t_trig=1, trigType=0, outbus,
		atk=0.01, dec=0.1, slev=0.5, rel=0.6;
	var in, env, trig;
	in = In.kr(inbus);
	trig = Select.kr(
		which:trigType,
		array:[
			t_trig,
			in
		]
	);
	env = EnvGen.kr(Env.adsr(atk,dec,slev,rel),trig);
	Out.kr(outbus, env);
}).add;

SynthDef(\gen, {
	arg	inbusAmp, inbusFreq, inbusWidth, outbus,
		sigType=0, ampType=0, freqType=0, widthType=0,
		amp=1, freqScale=1, baseFreq=440, widthScale=1, baseWidth=0;
	var sig, ampCtrl, freqCtrl, widthCtrl;

	ampCtrl = Select.kr(
		which:ampType,
		array:[
			amp,
			In.kr(inbusAmp)
		]
	);

	freqCtrl = Select.kr(
		which:freqType,
		array:[
			baseFreq,
			In.kr(inbusFreq) * freqScale + baseFreq
		]
	);

	widthCtrl = Select.kr(
		which:widthType,
		array:[
			baseWidth,
			In.kr(inbusWidth) * widthScale + baseWidth
		]
	);

	sig = Select.ar(
		which:sigType,
		array:[
			SinOsc.ar(freqCtrl, 0, ampCtrl),
			VarSaw.ar(freqCtrl, 0, widthCtrl, ampCtrl),
			Pulse.ar(freqCtrl, widthCtrl, ampCtrl),
			WhiteNoise.ar(ampCtrl),
			PinkNoise.ar(ampCtrl),
			BrownNoise.ar(ampCtrl)
		]
	);
	Out.ar(outbus, sig);
}).add;

SynthDef(\filt, {
	arg	inbus1, inbus2, inbus3, inbusCf,
		freqScale=3000, baseFreq=40, filtType=0, outbus;
	var in, cf, sig;
	in = [ In.ar(inbus1), In.ar(inbus2), In.ar(inbus3) ].sum;
	cf = In.kr(inbusCf) * freqScale + baseFreq;
	sig = Select.ar(
		which:filtType,
		array:[
			in,
			LPF.ar(in, cf),
			HPF.ar(in, cf)
		]
	);
	Out.ar(outbus, sig);
}).add;

SynthDef(\out, {
	arg	inbus, inbusPan, outbus=0,
		pan=0, panScale=2, basePan=(-1), panType=0,
		revTime=2.3, mix=0.07, amp=0.75;
	var in, pansig, inPan, sig, verbsig, outsig;
	in = In.ar(inbus);
	inPan = In.kr(inbusPan) * panScale + basePan;
	pansig = Select.kr(
		which:panType,
		array:[
			pan,
			inPan
		]
	);
	sig = Pan2.ar(in, pansig);

	verbsig = sig;
	outsig = 0;
	15.do({
		verbsig = AllpassN.ar(verbsig, 0.06, {Rand(0.001, 0.06)}!2, revTime);
		verbsig = LPF.ar(verbsig, 4500);
		outsig = outsig + verbsig;
	});
	outsig = XFade2.ar(sig, outsig, mix.linlin(0,1,-1,1), amp);
	Out.ar(outbus, outsig);
}).add;

//instantiate Groups, one for each SynthDef. Placing Synths in their respective Groups will ensure a proper Node order and a proper order-of-execution.
	~modGrp = Group.new;
	~trigGrp = Group.after(~modGrp);
	~envGrp = Group.after(~trigGrp);
	~genGrp = Group.after(~envGrp);
	~filtGrp = Group.after(~genGrp);
	~masterGrp = Group.after(~filtGrp);

//Free any busses that may already be allocated
	s.newBusAllocators;

//and allocate a number of control/audio busses for signal input/output routing.
	~modOut1 = Bus.control(s);
	~modOut2 = Bus.control(s);
	~modOut3 = Bus.control(s);
	~trigOut = Bus.control(s);
	~envOut = Bus.control(s);
	~genOut1 = Bus.audio(s);
	~genOut2 = Bus.audio(s);
	~genOut3 = Bus.audio(s);
	~filtOut = Bus.audio(s);
)

//After setting up our SynthDefs, Groups, and Busses, we can instantiate our Synths:
(
~modSynth1 = Synth(
	\mod,
	[
		\outbus, ~modOut1,
		\modType, 2, //low freq sine wave
		\freq, 0.2 //oscillating between 0 and 1 every 5 seconds
	],
	~modGrp);

~trigSynth = Synth(
	\trig,
	[
		\inbus, ~modOut1,

		//generate impulses at a frequency ranging
		//from 0.1 to 4.1
		\freqScale, 4,
		\baseFreq, 0.1,
		\trigType, 1,
		\outbus, ~trigOut
	],
	~trigGrp);

~envSynth = Synth(
	\adsr,
	[
		\inbus, ~trigOut,
		\trigType, 1, //use upstream impulses to trigger adsr
		\outbus, ~envOut
	],
	~envGrp);

~genSynth = Synth(
	\gen,
	[
		//read amplitude from adsr
		\inbusAmp, ~envOut,

		//read freq and width from upstream LF sine wave
		\inbusFreq, ~modOut1,
		\inbusWidth, ~modOut1,

		\outbus, ~genOut1,

		//pulse wave
		\sigType, 2,

		//type 1 means read from an upstream signal
		//instead of a manually updated value
		\ampType, 1,
		\freqType, 1,
		\widthType, 1,

		//scale & shift values (essentially mul & add)
		\freqScale, 200,
		\baseFreq, 50,
		\widthScale, 0.9,
		\baseWidth, 0.05
	], ~genGrp);

~filtSynth = Synth(
	\filt,
	[
		\inbus1, ~genOut1,
		\inbus2, ~genOut2,
		\inbus3, ~genOut3,

		//reads cutoff freq from LF sine
		\inbusCf, ~envOut,

		//lowpass filter (2 = hipass)
		\filtType, 1,
		\outbus, ~filtOut
	],
	~filtGrp);

~masterSynth = Synth(
	\out,
	[
		\inbus, ~filtOut,

		//pan position reads from LF sine as well
		\inbusPan, ~modOut1,

		\outbus, 0,
		\panType, 1,
		\mix, 0.12,
		\revTime, 3
	],
	~masterGrp);
)

//Now, the Fun Part: updating arguments to simulate knob turning and bus re-routing

//hold adsr open for longer (fewer triggers accepted)
~trigSynth.set(\len, 0.5);

//longer attack time
~envSynth.set(\atk, 0.5);

//longer release time
~envSynth.set(\rel, 0.5);

//hipass filter instead of lowpass filter:
~filtSynth.set(\filtType, 2);

//no filtering:
~filtSynth.set(\filtType, 0);

//back to lowpass:
~filtSynth.set(\filtType, 1);

//as an alternative, you can bypass filtering by routing the generated waveform directly to the master output:
~genSynth.set(\outbus, ~filtOut);

//but let's re-route it back to the filter Synth's input:
~genSynth.set(\outbus, ~genOut1);

//sound source = sine wave
~genSynth.set(\sigType, 0);

//variable duty saw
~genSynth.set(\sigType, 1);

//brown noise
~genSynth.set(\sigType, 5);

//return to a pulse wave with short envelopes & more frequent triggers (run these simultaneously, if you like):
(
~genSynth.set(\sigType, 2);
~trigSynth.set(\len, 0.1);
~envSynth.set(\atk, 0.02);
~envSynth.set(\rel, 0.1);
)

//change frequency scaling & shifting
~genSynth.set(\freqScale, 1000, \baseFreq, 40);

//even higher
~genSynth.set(\freqScale, 3000);

//select an LF saw modulator instead of LF sine. Recall that multiple downstream Synths currently rely on this modulator, ~modSynth1, for control sources (audio rate pulse width, pulse frequency, impulse frequency, and pan control). So when the modulator type is updated, numerous downstream sounds will change (e.g. frequence will now ramp up, and the sound will pan from left to right, and so on):
~modSynth1.set(\modType, 3);

//try modulating by linearly interpolated noise, picking 10 new values per second
~modSynth1.set(\modType, 1, \freq, 10);

//sample & hold noise, even faster
~modSynth1.set(\modType, 0, \freq, 50);

//return to a saw wave modulator at a slow frequency, and lower the pulse wave's frequency too:
(
~modSynth1.set(\modType, 3, \freq, 0.2);
~genSynth.set(\freqScale, 300, \baseFreq, 70);
)

//more triggers, and a shorter release
~trigSynth.set(\freqScale, 10, \baseFreq, 3, \len, 0.04);

//use irregular impulses (Dust.kr) instead of Impulse.kr
~trigSynth.set(\trigType, 2);

//Let's make a new modulator (a control-rate sine wave) and use it to control the pan position. We'll write this modulator's output to a separate control bus. We'll also make sure to put it in the modulator Group. Check the node tree to see for yourself that we've created a new module.
~modSynth2 = Synth(\mod, [\outbus, ~modOut2, \modType, 2, \freq, 1], ~modGrp);

//pan sinusoidally (previously panning was done via LF saw wave).
~masterSynth.set(\inbusPan, ~modOut2);

//And hey, why not, lets control the pulse wave's frequency with this new sine wave too:
~genSynth.set(\inbusFreq, ~modOut2);

//Please experiment with adding your own "set" commands and/or creating new modules (Synths). Better yet, start designing your own modular synthesizer!

s.freeAll;