//Basic window & GUI creation:

//A window's bounds are specified using the Rect object. The first two numbers, 100 and 100, correspond to the x/y distance between the bottom left corner of your screen and the bottom left corner of the window. The other two numbers specify the width and height of the window, respectively.
w = Window.new("Test Window", Rect.new(100,100,500,800)).front;

//GUI objects, on the other hand, interpret the first two numbers of Rect as the x/y distance between the top left corner of their parent view/window and their own top left corner. The third and fourth numbers determine width and height.
Slider.new(w, Rect.new(100,100,40,400));

//close the window
w.close;





//FlowLayout
w = Window.new("Test Window", Rect.new(100,100,500,800)).front;

//FlowLayout can be assigned to the decorator of any View and essentially locks your GUI to a grid. The user specifies the bounds of the FlowLayout (usually equal to the bounds of the View it is being used with), as well as a margin and a gap, both specified using the Point object. Margin determines the distance between GUI objects and the edge of the View, while gap determines the distance between adjacent GUI objects.
w.view.decorator = FlowLayout.new(w.view.bounds, Point.new(10,10), Point.new(10,10));

//Once a FlowLayout has been applied, only the dimensions of a GUI object need to be specified, since the FlowLayout determines the position and spacing of the GUI. Make three knobs, for example:
Knob.new(w, Point.new(40,40));
Knob.new(w, Point.new(40,40));
Knob.new(w, Point.new(40,40));

//Notice that these knobs are each 40x40 pixels, they are 10 pixels from the edge, both horizontally and vertically, and they are 10 pixels apart from one another.

//The following statement is essentially a "new line" command to FlowLayout
w.view.decorator.nextLine;

//Notice how this knob now appears on the next line, 10 pixels below the initial row of knobs
Knob.new(w, Point.new(40,40));

//There is a syntactical shortcut for Point:
Point.new(10,10);

//is the same as
10@10

//I am in the habit of using the shortened syntax and I suggest you do the same.

w.close;




//Views within Views: CompositeView

//CompositeView acts as a subview within a larger view. This is useful for conceptualizing a complex interface.
(
w = Window.new("Test Window", Rect.new(100,100,500,800)).front;
~panel = CompositeView.new(w, Rect.new(150,150,200,300));
~panel.background = Color.rand; //random color
~panel.decorator = FlowLayout.new(~panel.bounds, 10@10, 10@10);
)

//FlowLayout has been applied to the CompositeView, so it respects the given margin and gap arguments.
Knob.new(~panel, 40@40);

//this won't work on the window, since we haven't applied FlowLayout to the window's View. With no FlowLayout, SC interprets a Point as specifying the dimensions of the GUI and will place it in the upper left corner.
Knob.new(w, 40@40);

//Of course we can still specify a GUI using Rect on the window
Knob.new(w, Rect.new(60,60,50,50));

//Any GUI (CompositeView, Knob, etc) can be moved by specifying a new bounds:
~panel.bounds = Rect.new(50,350,200,300);

w.close;



//Quick GUI via Iteration

//Let's start by making an Array of 12 nils
~knobArray = Array.newClear(12);

//And our Window and CompositeView from the previous example:
(
w = Window.new("Test Window", Rect.new(100,100,500,800)).front;
~panel = CompositeView.new(w, Rect.new(150,150,200,300));
~panel.background = Color.rand; //random color
~panel.decorator = FlowLayout.new(~panel.bounds, 10@10, 10@10);
)

//"do" takes a single function as an argument and will evaluate that function as many times as its integer receiver. if an argument is used with "do", it will be interpreted as the current iteration, starting with zero. So, the following function will be evaluated 12 times. On the first pass, the argument "item" is equal to 0, on the second pass, item = 1, etc. On the last iterative pass, item = 11.
(
12.do({
	arg item;
	//Essentially, fill the Array with knobs.
	~knobArray[item] = Knob.new(~panel, 40@40);
});
)

//Because we stored these knobs in an Array, we can refer to a specific knob:
~knobArray[7].value = 0.5;

//We can randomize the values of all knobs like this:
(
12.do({
	arg item;
	~knobArray[item].value = rrand(0.0, 1.0);
});
)

//Or set all the knobs to the same value:
(
12.do({
	arg item;
	~knobArray[item].value = 0.2;
});
)