/*
I have made a few adjustments to the preexsting code. In particular:

1. A trigger can now be elongated via Trig.kr in the \adsr SynthDef as well as the \trig SynthDef. Previously, this could only be done in the \trig SynthDef. The reason I've added this feature is this: when opting for manual triggers (i.e. setting the \adsr Synth's \trigType argument to 0 and taking triggers from the t_trig argument instead of an upstream signal), there is no way to elongate the trigger and therefore no way to sustain the adsr's sustain portion. By sending the trigger through a Trig.kr, I can now elongate the manually received trigger and sustain the adsr envelope without having to use the upstream \trig SynthDef.

2. I've added a peak level argument to the \adsr SynthDef (named "peak"). This serves as an amplitude scalar for the adsr output.

Run this code first, and see the last section of code for examples of autonomous control, algorithmic composition, live coding, realtime audio manipulation, or whatever you'd like to call it. -EF
*/

s.makeGui;

(
ServerTree.removeAll;

~serverSetup = {
SynthDef(\mod, {
	arg freq=8, modType=0, outbus;
	var sig;
	sig = Select.kr(
		which:modType,
		array:[
			LFDNoise0.kr(freq).range(0,1),
			LFDNoise1.kr(freq).range(0,1),
			SinOsc.kr(freq).range(0,1),
			LFSaw.kr(freq).range(0,1),
			LFTri.kr(freq).range(0,1)
		]
	);
	Out.kr(outbus, sig);
}).add;

SynthDef(\trig, {
	arg inbus, freqScale=1, baseFreq=0, trigType=0, len=0.2, outbus;
	var in, sig;
	in = In.kr(inbus) * freqScale + baseFreq;
	sig = Select.kr(
		which:trigType,
		array:[
			Impulse.kr(baseFreq),
			Impulse.kr(in),
			Dust.kr(in)
		]
	);
	sig = Trig.kr(sig, len);
	Out.kr(outbus, sig);
}).add;

SynthDef(\adsr, {
	arg	inbus, t_trig=1, trigType=0, outbus,
		atk=0.01, dec=0.1, slev=0.5, rel=0.2, peak=1, len=0.001;
	var in, env, trig;
	in = In.kr(inbus);
	trig = Select.kr(
		which:trigType,
		array:[
			t_trig,
			in
		]
	);
	trig = Trig.kr(trig, len);
	env = EnvGen.kr(Env.adsr(atk,dec,slev,rel,peak),trig);
	Out.kr(outbus, env);
}).add;

SynthDef(\gen, {
	arg	inbusAmp, inbusFreq, inbusWidth, outbus,
		sigType=0, ampType=0, freqType=0, widthType=0,
		amp=1, freqScale=1, baseFreq=440, widthScale=1, baseWidth=0;
	var sig, ampCtrl, freqCtrl, widthCtrl;

	ampCtrl = Select.kr(
		which:ampType,
		array:[
			amp,
			In.kr(inbusAmp)
		]
	);

	freqCtrl = Select.kr(
		which:freqType,
		array:[
			baseFreq,
			In.kr(inbusFreq) * freqScale + baseFreq
		]
	);

	widthCtrl = Select.kr(
		which:widthType,
		array:[
			baseWidth,
			In.kr(inbusWidth) * widthScale + baseWidth
		]
	);
	sig = Select.ar(
		which:sigType,
		array:[
			SinOsc.ar(freqCtrl, 0, ampCtrl),
			VarSaw.ar(freqCtrl, 0, widthCtrl, ampCtrl),
			Pulse.ar(freqCtrl, widthCtrl, ampCtrl),
			WhiteNoise.ar(ampCtrl),
			PinkNoise.ar(ampCtrl),
			BrownNoise.ar(ampCtrl)
		]
	);
	Out.ar(outbus, sig);
}).add;

SynthDef(\filt, {
	arg	inbus1, inbus2, inbus3, inbusCf,
		freqScale=3000, baseFreq=40, filtType=0, outbus;
	var in, cf, sig;
	in = [ In.ar(inbus1), In.ar(inbus2), In.ar(inbus3) ].sum;
	cf = In.kr(inbusCf) * freqScale + baseFreq;
	sig = Select.ar(
		which:filtType,
		array:[
			in,
			LPF.ar(in, cf),
			HPF.ar(in, cf)
		]
	);
	Out.ar(outbus, sig);
}).add;

SynthDef(\out, {
	arg	inbus, inbusPan, outbus=0,
		pan=0, panScale=2, basePan=(-1), panType=0,
		revTime=2.3, mix=0.07, amp=0.75;
	var in, pansig, inPan, sig, verbsig, outsig;
	in = In.ar(inbus);
	inPan = In.kr(inbusPan) * panScale + basePan;
	pansig = Select.kr(
		which:panType,
		array:[
			pan,
			inPan
		]
	);
	sig = Pan2.ar(in, pansig);

	verbsig = sig;
	outsig = 0;
	15.do({
		verbsig = AllpassN.ar(verbsig, 0.06, {Rand(0.001, 0.06)}!2, revTime);
		verbsig = LPF.ar(verbsig, 4500);
		outsig = outsig + verbsig;
	});
	outsig = XFade2.ar(sig, outsig, mix.linlin(0,1,-1,1), amp);
	Out.ar(outbus, outsig);
}).add;

	~modGrp = Group.new;
	~trigGrp = Group.after(~modGrp);
	~envGrp = Group.after(~trigGrp);
	~genGrp = Group.after(~envGrp);
	~filtGrp = Group.after(~genGrp);
	~masterGrp = Group.after(~filtGrp);

	s.newBusAllocators; //free all busses before allocating new ones
	~modOut1 = Bus.control(s);
	~modOut2 = Bus.control(s);
	~modOut3 = Bus.control(s);
	~trigOut = Bus.control(s);
	~envOut = Bus.control(s);
	~genOut1 = Bus.audio(s);
	~genOut2 = Bus.audio(s);
	~genOut3 = Bus.audio(s);
	~filtOut = Bus.audio(s);
};

ServerTree.add(~serverSetup);
s.reboot;
)

(
//make Synths
~modSynth1 = Synth(
	\mod,
	[
		\outbus, ~modOut1,
		\modType, 0,
		\freq, 1
	],
	~modGrp);

~modSynth2 = Synth(
	\mod,
	[
		\outbus, ~modOut2,
		\modType, 1,
		\freq, 1
	],
	~modGrp);

~modSynth3 = Synth(
	\mod,
	[
		\outbus, ~modOut3,
		\modType, 2,
		\freq, 1
	],
	~modGrp);

~trigSynth = Synth(
	\trig,
	[
		\inbus, ~modOut1,
		\freqScale, 1,
		\baseFreq, 1,
		\trigType, 1,
		\outbus, ~trigOut
	],
	~trigGrp);

~envSynth = Synth(
	\adsr,
	[
		\inbus, ~trigOut,
		\trigType, 1,
		\outbus, ~envOut
	],
	~envGrp);

~genSynth1 = Synth(
	\gen,
	[
		\inbusAmp, ~envOut,
		\inbusFreq, ~modOut1,
		\inbusWidth, ~modOut1,
		\outbus, ~genOut1,
		\sigType, 0,
		\ampType, 1,
		\freqType, 1,
		\widthType, 1,
		\freqScale, 200,
		\baseFreq, 50,
		\widthScale, 0.9,
		\baseWidth, 0.05
	], ~genGrp);

~genSynth2 = Synth(
	\gen,
	[
		\inbusAmp, ~envOut,
		\inbusFreq, ~modOut1,
		\inbusWidth, ~modOut1,
		\outbus, ~genOut2,
		\sigType, 0,
		\ampType, 0,
		\amp, 0,
		\freqType, 1,
		\widthType, 1,
		\freqScale, 200,
		\baseFreq, 50,
		\widthScale, 0.9,
		\baseWidth, 0.05
	], ~genGrp);

~genSynth3 = Synth(
	\gen,
	[
		\inbusAmp, ~envOut,
		\inbusFreq, ~modOut1,
		\inbusWidth, ~modOut1,
		\outbus, ~genOut3,
		\sigType, 0,
		\ampType, 0,
		\amp, 0,
		\freqType, 1,
		\widthType, 1,
		\freqScale, 200,
		\baseFreq, 50,
		\widthScale, 0.9,
		\baseWidth, 0.05
	], ~genGrp);

~filtSynth = Synth(
	\filt,
	[
		\inbus1, ~genOut1,
		\inbus2, ~genOut2,
		\inbus3, ~genOut3,
		\inbusCf, ~envOut,
		\filtType, 1,
		\outbus, ~filtOut
	],
	~filtGrp);

~masterSynth = Synth(
	\out,
	[
		\inbus, ~filtOut,
		\inbusPan, ~modOut1,
		\outbus, 0,
		\panType, 1,
		\mix, 0.12,
		\revTime, 3,
		\amp, 0
	],
	~masterGrp);




//make GUI
w = Window("modular synth", Rect(100,100,690,630)).front;

~modPanel = CompositeView(w,Rect(0,0,210,280)).background_(Color.rand);
~modPanel.decorator = FlowLayout(~modPanel.bounds, 3@3, 3@9);

StaticText(~modPanel, 200@24)
.string_("Modulators")
.font_(Font("Helvetica", 20))
.align_(\center);

StaticText(~modPanel, 50@24)
.string_("type")
.align_(\center)
.font_(Font("Helvetica", 20));

PopUpMenu(~modPanel, 150@24)
.items_([
	"samp & hold noise",
	"linear noise",
	"LF Sine",
	"LF Saw",
	"LF Tri"
	])
.action_({
	arg thisMenu;
	~modSynth1.set(\modType, thisMenu.value);
});

StaticText(~modPanel, 30@24)
.string_(" hz")
.align_(\center)
.font_(Font("Helvetica", 20));

Knob(~modPanel, 36@24)
.action_({
	arg thisKnob;
	~modSynth1.set(\freq, thisKnob.value.linexp(0,1,0.05,20));
})
.value_(0.5);

StaticText(~modPanel, 60@24)
.string_("    out")
.font_(Font("Helvetica", 20));


PopUpMenu(~modPanel, 68@24)
.items_([
	"mod1",
	"mod2",
	"mod3"
	])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~modSynth1.set(\outbus, ~modOut1)}
	{thisMenu.value == 1}
	{~modSynth1.set(\outbus, ~modOut2)}
	{thisMenu.value == 2}
	{~modSynth1.set(\outbus, ~modOut3)}
	{true}{nil};
});

StaticText(~modPanel, 200@12)
.string_("");

StaticText(~modPanel, 50@24)
.string_("type")
.align_(\center)
.font_(Font("Helvetica", 20));

PopUpMenu(~modPanel, 150@24)
.items_([
	"samp & hold noise",
	"linear noise",
	"LF Sine",
	"LF Saw",
	"LF Tri"
	])
.value_(1)
.action_({
	arg thisMenu;
	~modSynth2.set(\modType, thisMenu.value);
});

StaticText(~modPanel, 30@24)
.string_(" hz")
.align_(\center)
.font_(Font("Helvetica", 20));

Knob(~modPanel, 36@24)
.action_({
	arg thisKnob;
	~modSynth2.set(\freq, thisKnob.value.linexp(0,1,0.05,20));
})
.value_(0.5);

StaticText(~modPanel, 60@24)
.string_("    out")
.font_(Font("Helvetica", 20));


PopUpMenu(~modPanel, 68@24)
.items_([
	"mod1",
	"mod2",
	"mod3"
	])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~modSynth2.set(\outbus, ~modOut1)}
	{thisMenu.value == 1}
	{~modSynth2.set(\outbus, ~modOut2)}
	{thisMenu.value == 2}
	{~modSynth2.set(\outbus, ~modOut3)}
	{true}{nil};
})
.value_(1);

StaticText(~modPanel, 200@12)
.string_("");

StaticText(~modPanel, 50@24)
.string_("type")
.align_(\center)
.font_(Font("Helvetica", 20));

PopUpMenu(~modPanel, 150@24)
.items_([
	"samp & hold noise",
	"linear noise",
	"LF Sine",
	"LF Saw",
	"LF Tri"
	])
.value_(2)
.action_({
	arg thisMenu;
	~modSynth3.set(\modType, thisMenu.value);
});

StaticText(~modPanel, 30@24)
.string_(" hz")
.align_(\center)
.font_(Font("Helvetica", 20));

Knob(~modPanel, 36@24)
.action_({
	arg thisKnob;
	~modSynth3.set(\freq, thisKnob.value.linexp(0,1,0.05,20));
})
.value_(0.5);

StaticText(~modPanel, 60@24)
.string_("    out")
.font_(Font("Helvetica", 20));


PopUpMenu(~modPanel, 68@24)
.items_([
	"mod1",
	"mod2",
	"mod3"
	])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~modSynth3.set(\outbus, ~modOut1)}
	{thisMenu.value == 1}
	{~modSynth3.set(\outbus, ~modOut2)}
	{thisMenu.value == 2}
	{~modSynth3.set(\outbus, ~modOut3)}
	{true}{nil};
})
.value_(2);

~trigPanel = CompositeView(w,Rect(0,280,210,190)).background_(Color.rand);
~trigPanel.decorator = FlowLayout(~trigPanel.bounds, 3@3, 3@12);

StaticText(~trigPanel, 200@24)
.string_("Triggers")
.font_(Font("Helvetica", 20))
.align_(\center);

StaticText(~trigPanel, 50@24)
.string_("type")
.align_(\center)
.font_(Font("Helvetica", 20));

PopUpMenu(~trigPanel, 150@24)
.items_([
	"basic impulse",
	"signal impulse",
	"signal dust"
	])
.action_({
	arg thisMenu;
	~trigSynth.set(\trigType, thisMenu.value);
})
.value_(1);

StaticText(~trigPanel, 60@28)
.string_("base Hz")
.align_(\center)
.font_(Font("Helvetica", 14));

Knob(~trigPanel, 36@24)
.action_({
	arg thisKnob;
	~trigSynth.set(\baseFreq, thisKnob.value.linexp(0,1,0.05,20));
})
.value_(0.5);

StaticText(~trigPanel, 60@20)
.string_("Hz range")
.align_(\center)
.font_(Font("Helvetica", 14));

Knob(~trigPanel, 36@24)
.action_({
	arg thisKnob;
	~trigSynth.set(\freqScale, thisKnob.value.linlin(0,1,0,20));
})
.value_(0.05);

StaticText(~trigPanel, 50@28)
.string_("trig dur")
.align_(\center)
.font_(Font("Helvetica", 14));

Slider(~trigPanel, 150@20)
.action_({
	arg thisSlider;
	~trigSynth.set(\len, thisSlider.value.linexp(0,1,0.005,10));
})
.value_(0.2.explin(0.005,10,0,1));


StaticText(~trigPanel, 30@28)
.string_("in")
.align_(\center)
.font_(Font("Helvetica", 16));

PopUpMenu(~trigPanel, 68@24)
.items_([
	"mod1",
	"mod2",
	"mod3"
	])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~trigSynth.set(\inbus, ~modOut1)}
	{thisMenu.value == 1}
	{~trigSynth.set(\inbus, ~modOut2)}
	{thisMenu.value == 2}
	{~trigSynth.set(\inbus, ~modOut3)}
	{true}{nil};
});

StaticText(~trigPanel, 30@28)
.string_("out")
.align_(\center)
.font_(Font("Helvetica", 16));

PopUpMenu(~trigPanel, 50@24)
.items_([
	"trig"
	])
.action_({
	arg thisMenu;
	~trigSynth.set(\outbus, ~trigOut)
});

~adsrPanel = CompositeView(w,Rect(0,470,210,160)).background_(Color.rand);
~adsrPanel.decorator = FlowLayout(~adsrPanel.bounds, 3@3, 3@12);

StaticText(~adsrPanel, 200@24)
.string_("ADSR Envelope")
.font_(Font("Helvetica", 20))
.align_(\center);

StaticText(~adsrPanel, 46@24)
.string_("type")
.align_(\center)
.font_(Font("Helvetica", 20));

PopUpMenu(~adsrPanel, 80@24)
.items_([
	"manual",
	"signal"
	])
.action_({
	arg thisMenu;
	~envSynth.set(\trigType, thisMenu.value);
})
.value_(1);

StaticText(~adsrPanel, 24@24).string_("");

Button(~adsrPanel, 24@24)
.states_([["T"]])
.action_({~envSynth.set(\t_trig, 1)});

StaticText(~adsrPanel, 20@28)
.string_("in")
.align_(\center)
.font_(Font("Helvetica", 16));

PopUpMenu(~adsrPanel, 68@24)
.items_([
	"mod1",
	"mod2",
	"mod3",
	"trig"
	])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~envSynth.set(\inbus, ~modOut1)}
	{thisMenu.value == 1}
	{~envSynth.set(\inbus, ~modOut2)}
	{thisMenu.value == 2}
	{~envSynth.set(\inbus, ~modOut3)}
	{thisMenu.value == 3}
	{~envSynth.set(\inbus, ~trigOut)}
	{true}{nil};
})
.value_(3);

StaticText(~adsrPanel, 40@28)
.string_("   out")
.align_(\center)
.font_(Font("Helvetica", 16));

PopUpMenu(~adsrPanel, 63@24)
.items_([
	"adsr"
	])
.action_({
	arg thisMenu;
	~envSynth.set(\outbus, ~envOut)
});

StaticText(~adsrPanel, 38@30).string_("asdr").align_(\center);

NumberBox(~adsrPanel, 38@24)
.action_({
	arg thisBox;
	~envSynth.set(\atk, thisBox.value);
})
.value_(0.01)
.clipLo_(0.001)
.clipHi_(20);

NumberBox(~adsrPanel, 38@24)
.action_({
	arg thisBox;
	~envSynth.set(\dec, thisBox.value);
})
.value_(0.1)
.clipLo_(0.001)
.clipHi_(20);

NumberBox(~adsrPanel, 38@24)
.action_({
	arg thisBox;
	~envSynth.set(\slev, thisBox.value);
})
.value_(0.5)
.clipLo_(0.0)
.clipHi_(1.0);

NumberBox(~adsrPanel, 38@24)
.action_({
	arg thisBox;
	~envSynth.set(\rel, thisBox.value);
})
.value_(0.2)
.clipLo_(0.001)
.clipHi_(20);

~genPanel = CompositeView(w,Rect(210,0,240,500)).background_(Color.rand);
~genPanel.decorator = FlowLayout(~genPanel.bounds, 3@3, 3@3);

StaticText(~genPanel, 240@24)
.string_("Signal Generators")
.font_(Font("Helvetica", 20))
.align_(\center);

StaticText(~genPanel, 30@24)
.string_("type");

PopUpMenu(~genPanel, 200@24)
.items_([\sine, \varsaw, \pulse, \white, \pink, \brown])
.action_({
	arg thisMenu;
	~genSynth1.set(\sigType, thisMenu.value);
});

StaticText(~genPanel, 30@24)
.string_("amp");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth1.set(\ampType, thisMenu.value);
})
.value_(1);

PopUpMenu(~genPanel, 90@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth1.set(\inbusAmp, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth1.set(\inbusAmp, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth1.set(\inbusAmp, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth1.set(\inbusAmp, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth1.set(\inbusAmp, ~envOut)}
})
.value_(4);

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth1.set(\amp, thisKnob.value);
})
.value_(1);

StaticText(~genPanel, 30@24)
.string_("freq");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth1.set(\freqType, thisMenu.value);
})
.value_(1);

PopUpMenu(~genPanel, 62@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth1.set(\inbusFreq, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth1.set(\inbusFreq, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth1.set(\inbusFreq, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth1.set(\inbusFreq, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth1.set(\inbusFreq, ~envOut)}
})
.value_(0);

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth1.set(\baseFreq, thisKnob.value.linexp(0,1,20,16000));
})
.value(50.explin(20,16000,0,1));

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth1.set(\freqScale, thisKnob.value.linlin(0,1,0,5000));
})
.value_(200.linlin(0,5000,0,1));

StaticText(~genPanel, 30@24)
.string_("wdth");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth1.set(\widthType, thisMenu.value);
})
.value_(1);

PopUpMenu(~genPanel, 62@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth1.set(\inbusWidth, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth1.set(\inbusWidth, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth1.set(\inbusWidth, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth1.set(\inbusWidth, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth1.set(\inbusWidth, ~envOut)}
})
.value_(0);

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth1.set(\baseWidth, thisKnob.value.clip(0.001,0.999));
})
.value_(0.05);

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth1.set(\widthScale, thisKnob.value)
})
.value(0.9);

StaticText(~genPanel, 30@24)
.string_("out");

PopUpMenu(~genPanel, 62@24)
.items_([\gen1, \gen2, \gen3])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth1.set(\outbus, ~genOut1)}
	{thisMenu.value == 1}
	{~genSynth1.set(\outbus, ~genOut2)}
	{thisMenu.value == 2}
	{~genSynth1.set(\outbus, ~genOut3)};
});

StaticText(~genPanel, 210@24)
.string_("");

StaticText(~genPanel, 30@24)
.string_("type");

PopUpMenu(~genPanel, 200@24)
.items_([\sine, \varsaw, \pulse, \white, \pink, \brown])
.action_({
	arg thisMenu;
	~genSynth2.set(\sigType, thisMenu.value);
});

StaticText(~genPanel, 30@24)
.string_("amp");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth2.set(\ampType, thisMenu.value);
});

PopUpMenu(~genPanel, 90@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth2.set(\inbusAmp, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth2.set(\inbusAmp, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth2.set(\inbusAmp, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth2.set(\inbusAmp, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth2.set(\inbusAmp, ~envOut)}
});

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth2.set(\amp, thisKnob.value);
});

StaticText(~genPanel, 30@24)
.string_("freq");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth2.set(\freqType, thisMenu.value);
});

PopUpMenu(~genPanel, 62@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth2.set(\inbusFreq, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth2.set(\inbusFreq, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth2.set(\inbusFreq, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth2.set(\inbusFreq, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth2.set(\inbusFreq, ~envOut)}
});

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth2.set(\baseFreq, thisKnob.value.linexp(0,1,20,16000));
})
.value_(440.explin(20,16000,0,1));

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth2.set(\freqScale, thisKnob.value.linlin(0,1,0,5000));
})
.value_(0);

StaticText(~genPanel, 30@24)
.string_("wdth");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth2.set(\widthType, thisMenu.value);
});

PopUpMenu(~genPanel, 62@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth2.set(\inbusWidth, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth2.set(\inbusWidth, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth2.set(\inbusWidth, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth2.set(\inbusWidth, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth2.set(\inbusWidth, ~envOut)}
});

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth2.set(\baseWidth, thisKnob.value.clip(0.001,0.999));
})
.value_(0.05);

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth2.set(\widthScale, thisKnob.value)
})
.value(0.9);

StaticText(~genPanel, 30@24)
.string_("out");

PopUpMenu(~genPanel, 62@24)
.items_([\gen1, \gen2, \gen3])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth2.set(\outbus, ~genOut1)}
	{thisMenu.value == 1}
	{~genSynth2.set(\outbus, ~genOut2)}
	{thisMenu.value == 2}
	{~genSynth2.set(\outbus, ~genOut3)};
})
.value_(1);

StaticText(~genPanel, 210@24)
.string_("");

StaticText(~genPanel, 30@24)
.string_("type");

PopUpMenu(~genPanel, 200@24)
.items_([\sine, \varsaw, \pulse, \white, \pink, \brown])
.action_({
	arg thisMenu;
	~genSynth3.set(\sigType, thisMenu.value);
});

StaticText(~genPanel, 30@24)
.string_("amp");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth3.set(\ampType, thisMenu.value);
});

PopUpMenu(~genPanel, 90@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth3.set(\inbusAmp, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth3.set(\inbusAmp, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth3.set(\inbusAmp, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth3.set(\inbusAmp, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth3.set(\inbusAmp, ~envOut)}
});

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth3.set(\amp, thisKnob.value);
});

StaticText(~genPanel, 30@24)
.string_("freq");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth3.set(\freqType, thisMenu.value);
});

PopUpMenu(~genPanel, 62@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth3.set(\inbusFreq, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth3.set(\inbusFreq, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth3.set(\inbusFreq, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth3.set(\inbusFreq, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth3.set(\inbusFreq, ~envOut)}
});

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth3.set(\baseFreq, thisKnob.value.linexp(0,1,20,16000));
})
.value_(440.explin(20,16000,0,1));

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth3.set(\freqScale, thisKnob.value.linlin(0,1,0,5000));
})
.value_(0);

StaticText(~genPanel, 30@24)
.string_("wdth");

PopUpMenu(~genPanel, 80@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~genSynth3.set(\widthType, thisMenu.value);
});

PopUpMenu(~genPanel, 62@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth3.set(\inbusWidth, ~modOut1)}
	{thisMenu.value == 1}
	{~genSynth3.set(\inbusWidth, ~modOut2)}
	{thisMenu.value == 2}
	{~genSynth3.set(\inbusWidth, ~modOut3)}
	{thisMenu.value == 3}
	{~genSynth3.set(\inbusWidth, ~trigOut)}
	{thisMenu.value == 4}
	{~genSynth3.set(\inbusWidth, ~envOut)}
});

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth3.set(\baseWidth, thisKnob.value.clip(0.001,0.999));
})
.value_(0.05);

Knob(~genPanel, 24@24)
.action_({
	arg thisKnob;
	~genSynth3.set(\widthScale, thisKnob.value)
})
.value(0.9);

StaticText(~genPanel, 30@24)
.string_("out");

PopUpMenu(~genPanel, 62@24)
.items_([\gen1, \gen2, \gen3])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~genSynth3.set(\outbus, ~genOut1)}
	{thisMenu.value == 1}
	{~genSynth3.set(\outbus, ~genOut2)}
	{thisMenu.value == 2}
	{~genSynth3.set(\outbus, ~genOut3)};
})
.value_(2);

StaticText(~genPanel, 200@24)
.string_("");

~filtPanel = CompositeView(w,Rect(210,500,240,130)).background_(Color.rand);
~filtPanel.decorator = FlowLayout(~filtPanel.bounds, 3@3, 3@9);

StaticText(~filtPanel, 240@24)
.string_("Filter")
.font_(Font("Helvetica", 20))
.align_(\center);

StaticText(~filtPanel, 30@24)
.string_("type");

PopUpMenu(~filtPanel, 200@24)
.items_([\none, \LPF, \HPF])
.action_({
	arg thisMenu;
	~filtSynth.set(\filtType, thisMenu.value);
})
.value_(1);

StaticText(~filtPanel, 30@24)
.string_("cf");

PopUpMenu(~filtPanel, 130@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~filtSynth.set(\inbusCf, ~modOut1)}
	{thisMenu.value == 1}
	{~filtSynth.set(\inbusCf, ~modOut2)}
	{thisMenu.value == 2}
	{~filtSynth.set(\inbusCf, ~modOut3)}
	{thisMenu.value == 3}
	{~filtSynth.set(\inbusCf, ~trigOut)}
	{thisMenu.value == 4}
	{~filtSynth.set(\inbusCf, ~envOut)}
})
.value_(4);

Knob(~filtPanel, 24@24)
.action_({
	arg thisKnob;
	~filtSynth.set(\baseFreq, thisKnob.value.linexp(0,1,20,15000));
})
.value_(40.explin(20,15000,0,1));

Knob(~filtPanel, 24@24)
.action_({
	arg thisKnob;
	~filtSynth.set(\freqScale, thisKnob.value.linlin(0,1,0,5000));
})
.value_(0.6);

~outPanel = CompositeView(w,Rect(450,0,240,630)).background_(Color.rand);
~outPanel.decorator = FlowLayout(~outPanel.bounds, 3@3, 3@3);

StaticText(~outPanel, 240@28)
.string_("Main Out")
.font_(Font("Helvetica", 20))
.align_(\center);

StaticText(~outPanel, 35@28)
.string_("amp")
.align_(\center)
.font_(Font("Helvetica", 14));

Slider(~outPanel, 190@20)
.action_({
	arg thisSlider;
	~masterSynth.set(\amp, thisSlider.value);
})
.value_(0);

StaticText(~outPanel, 50@28)
.string_("rev mix")
.align_(\center)
.font_(Font("Helvetica", 14));

Slider(~outPanel, 175@20)
.action_({
	arg thisSlider;
	~masterSynth.set(\mix, thisSlider.value);
})
.value_(0.12);

StaticText(~outPanel, 50@28)
.string_("rev time")
.align_(\center)
.font_(Font("Helvetica", 14));

Slider(~outPanel, 175@20)
.action_({
	arg thisSlider;
	~masterSynth.set(\mix, thisSlider.value.linexp(0,1,0.1,5));
})
.value_(3.explin(0.1,5,0,1));

StaticText(~outPanel, 30@28)
.string_("pan")
.align_(\center)
.font_(Font("Helvetica", 14));

PopUpMenu(~outPanel, 100@24)
.items_([\manual, \signal])
.action_({
	arg thisMenu;
	~masterSynth.set(\panType, thisMenu.value);
})
.value_(1);

PopUpMenu(~outPanel, 70@24)
.items_([\mod1, \mod2, \mod3, \trig, \adsr])
.action_({
	arg thisMenu;
	case
	{thisMenu.value == 0}
	{~masterSynth.set(\inbusPan, ~modOut1)}
	{thisMenu.value == 1}
	{~masterSynth.set(\inbusPan, ~modOut2)}
	{thisMenu.value == 2}
	{~masterSynth.set(\inbusPan, ~modOut3)}
	{thisMenu.value == 3}
	{~masterSynth.set(\inbusPan, ~trigOut)}
	{thisMenu.value == 4}
	{~masterSynth.set(\inbusPan, ~envOut)}
})
.value_(0);

RangeSlider(~outPanel, 234@30)
.action_({
	arg thisRanger;
	~masterSynth.set(\pan, (thisRanger.lo + (thisRanger.range/2)).linlin(0,1,-1,1));
	~masterSynth.set(\basePan, thisRanger.lo.linlin(0,1,-1,1));
	~masterSynth.set(\panScale, thisRanger.range);
});
)

//After loading SynthDefs, Busses, Groups, Synths, and GUI, we're ready to let the SC interpreter take over the compositional/generative process.

//The following clump will set some initial parameters that are necessary for autonomous control. Please read the comments carefully before proceeding!

(
//By default, we'll only be listening to a sine wave being produced by a Synth named ~genSynth1. We want to control the frequency (and perhaps other parameters) via executed code from the SC language-side, rather than control signals from upstream Synths on the server (LFNoise0, LFSaw, etc). Therefore, we'll set \freqType to 0, which means we'll determine our frequency from baseFreq and freqScale (user-supplied arguments, also GUI-controlled), rather than from an upstream control signal. We'll also set \freqScale to 0, which means \baseFreq is now the ONLY determining factor in the perceived pitch. In fact, now \baseFreq == perceived Hz.

//However, we do want to use the adsr envelope to control genSynth1's amplitude, so we'll make sure \ampType is set to 1 (which means DO read from an upstream control signal, in particular, the control signal being written to the Bus called ~envOut, which we specify for the argument \inbusAmp).

//If it wasn't clear already, it's syntactically legal to set multiple arguments in a single "set" statement. Also, don't forget that by default, the Main Output amplitude slider is at zero. Slide it to the right to hear sound.
~genSynth1.set(\freqType, 0, \freqScale, 0, \ampType, 1, \inbusAmp, ~envOut);

//We want to be able to trigger an adsr envelope manually, rather than relying on an upstream signal for triggers. So, we'll set \trigType to 0. Now, setting the argument "t_trig" equal to 1 will trigger the adsr envelope.
~envSynth.set(\trigType, 0);

//Fact: it is super fun to be able to manipulate sound as it's playing. So I'm defining some global variable here:

//a fundamental pitch
~fund = 100;

//harmonics to be multiplied by the fundamental
~harms = {[1,2,4,5,6].choose};

//a peak amplitude value for the adsr envelope
~peak = {exprand(0.1,1)};

//here's a Task stored in the global variable t. The Task contains a single internal function which contains two "set" commands and a 0.17-second wait time. This internal function will loop indefinitely until the external Task is stopped with the "stop" message.
t = Task({
	{
		//set the base frequency of the sine oscillator to the value of ~fund, and multiply it by the value stored in ~harms. By using the "value" message here, it becomes legal to set the value of ~fund, ~harms, and ~peak to be EITHER a number OR a Function, since "value" is defined for numbers as well as Functions. If, for example, ~fund is a number, then value will have no effect on the receiver. If ~fund is a function, then "value" will evaluate the function. This makes it very simple to use methods like "choose" (for picking a single item in an array) or operations like rrand and exprand.

		//in short, we set the frequency parameters,
		~genSynth1.set(\baseFreq, ~fund.value * ~harms.value);

		//set some adsr parameters, and trigger the envelope,
		~envSynth.set(\peak, ~peak.value, \len, exprand(0.001,0.025), \t_trig, 1);

		//then we wait 0.17 seconds,
		0.17.wait;

	}.loop //and loop this function.

}).start; //the Task starts up immediately after this code is evaluated. Give it a try!
)

//our fundamental is at 100Hz, and we're hearing either the 1st, 2nd, 4th, 5th, or 6th partial every 0.17 seconds. Since our Task is looping (i.e. re-evaluating) a function, we can substitute values for our global variables. Run the following single-line statements one at a time:

//transpose to the dominant
~fund = 150;

//transpose to the subdominant
~fund = 133.3;

//transpose back to the tonic with some flat 7th degree mixed in. Also, modify the possible harmonics. "wchoose" is a weighted choose and requires as its only argument an Array of weights, which must sum to 1. You can use the method "normalizeSum" on an Array to scale its contents so that they sum to 1.
~fund = {[100, 89.09].wchoose([0.6,0.4])}; ~harms = {[1,2,4,8].wchoose([15,20,30,45].normalizeSum)};

//more modifications - higher pitches
~fund = {[100,200,400,800].choose}; ~harms = {[1,2,3,4,5].choose};

//slightly different fundamental pitches and transposed up by a major 2nd
~fund = {[100,200,500,600].choose * 9/8};

//slightly quieter, subtle change in fundamental pitches
~fund = {[100,200,500,800].choose}; ~peak = {exprand(0.05,0.8)};

//fewer pitch choices
~fund = {[200,400,800].choose}; ~harms = {[2,3,4].choose};

//only octaves, even fewer pitch choices, even fewer loud notes
~fund = {[400,800].choose}; ~harms = {[0.5,1,2].choose}; ~peak = {exprand(0.01,0.7)};

//only octaves, very few pitch choices and very few loud notes
~fund = 400; ~harms = {[1,2].wchoose([0.2,0.8])}; ~peak = {exprand(0.005,0.5)};

//done
t.stop;

//Notice that I haven't created global variables for the waveform width control (not relevant for a sine wave, but you could switch to a pulse wave or variable duty saw wave), adsr attack time, release time, or the function's wait time (among numerous other parameters), but there's no reason not to do so! This is a fairly simple example with lots of room for exploration.

//It's fun to pick randomly from a list, and it's fun to generate randomly from a range of numbers, but I'm assuming many of you are wondering how to sequentially cycle through a list of values, or perhaps use a list/range of values in some other fashion. Well, you're in luck. There's an extensive family of objects called Patterns, which are specifically designed to algorithmically generate notes, values, events, and just about anything else you can think of.

//Arguably, the simplest Pattern is an object called "Pseq", which is short for "sequential pattern". Pseq takes an Array of values, and a number of repeats, for which you can specify a number or "inf" for an infinite number of repeats.

//For example, the following Pseq

Pseq([7,12,9],3);

//Would generate the following sequence of values:

7
12
9
7
12
9
7
12
9

//By using the method "asStream" on a Pattern, the Pattern is converted to a Routine. Once we've got a Routine on our hands, we can use the "next" method to step through the pattern. Run the following code to see for yourself:

//Convert to a Routine;
p = Pseq([7,12,9],3).asStream;

//Then run this line multiple times. "nil" will be returned once the Pseq is finished.
p.next;

//Algorithmically ontrolling Synth arguments within a Task becomes trivial with the use of Patterns. Evaluate and study the code below for an example in which I'm using Pseq to determine wait times:

(
~genSynth1.set(\freqType, 0, \freqScale, 0, \ampType, 1, \inbusAmp, ~envOut);
~envSynth.set(\trigType, 0);
~fund = 120;
~harms = {[1,2,4,8].choose};
~peak = {exprand(0.3,0.6)};

//I'm using simple numerical values (1, 2, 0.5, etc) and then multiplying the Array by 0.17 to scale the wait times - I find it much easier to conceptualize durations this way. I want this Pattern to repeat infinitely, so I'm using "inf", and I'm converting it to a Routine using "asStream".
~waitTime = Pseq([3,0.5,0.5,2,1,2,1,1,1,3,1]*0.17,inf).asStream;

t = Task({
	{
		~genSynth1.set(\baseFreq, ~fund.value * ~harms.value);
		~envSynth.set(\peak, ~peak.value, \len, exprand(0.001,0.025), \t_trig, 1);

		//All I have to do is call "next" on the Routine, and then call "wait" on the returned value. So easy!
		~waitTime.next.wait;
	}.loop
}).start;
)

//stop the Task when you're done
t.stop;