//Pbind: An Introduction

//Pbind is one of SC's extensive Pattern library. Specifically, Pbind generates Events by binding values to Symbols. There are many different types of Events, but the most common is the \note Event, which is simply a Synth instantiation (i.e. Synth.new). Pbind is therefore an alternative (and a superior one at that) to using Tasks or Routines to generate several Synth instantiations.

//We'll start by adding a mildly complex SynthDef.
s.boot;

(
SynthDef(\sawTone, {
	arg	freq=440, freqDev=1,
		widthMin=0.1, widthMax=0.35,
		useFilt=1, cf=5000, amp=0.6,
		atk=0.001, rel=0.4, curve=(-3), pan=0, out=0;

	var env, sig;

	//percussive envelope
	env = EnvGen.kr(Env.perc(atk,rel,1,curve), doneAction:2);

	//applied to eight unique VarSaw UGens with various random parameters
	sig = VarSaw.ar(
		{freq * rrand(freqDev.reciprocal, freqDev)}!8,
		{Rand(0.0,1.0)}!8,
		{ExpRand(widthMin,widthMax)}!8
	);

	//optionally filter the signal
	sig = Select.ar(useFilt, [sig, LPF.ar(sig, cf)]);

	//mix octophonic to mono
	sig = Mix(sig);

	//apply the envelope
	sig = sig * env;

	//stereo pan
	sig = Pan2.ar(sig, pan, amp);
	Out.ar(out, sig);
}).add;
)

//here's what it sounds like:
Synth(\sawTone);

//we've got a good amount of variety at our disposal:
(
Synth(
	\sawTone,
	[
		\freq, [33,45].choose.midicps,
		\amp, 0.3,
		\cf, 800,
		\freqDev, 1.1,
		\pan, rrand(-1.0,1.0),
		\rel, 1.2
	]
);
)

//Previously, we'd been using Tasks to automate the Synth generation process, like so:
(
t = Task({
	{
		Synth(
			\sawTone,
			[
				\freq, exprand(60,700),
				\cf, exprand(800,5000),
				\freqDev, rrand(1.01,1.06),
				\rel, 0.14,
				\pan, rrand(-1.0,1.0),
				\amp, exprand(0.1,0.5),
			]
		);
		exprand(0.05,0.8).wait;
	}.loop;
}).start;
)

t.stop;

//We'll now create this same algorithmic composition using Pbind instead of Task:

(
p = Pbind(

	//Pbind needs to know what type of Event it will be generating. In this case, we'll be generating "\note" Events (i.e. Synth instantiations).
	\type, 'note',

	//Since these are Synth instantiations, we need to know which SynthDef to use. This is specified with "\instrument"
	\instrument, 'sawTone',

	//"\dur" is a symbolic key that Pbind uses, regardless of Event type. This values specifies the time between Event generations, usually known as "delta times". In our previous Task, this was implemented by choosing a random value and using the "wait" method.

	//Also notice that, because we are using Patterns exclusivly (i.e. no Tasks or Routines), we do not have to use "asStream" to convert the Pattern to a Routine.
	\dur, Plprand(0.05, 0.8, inf),

	//Now, we simply specify patterns to control our Synth arguments (order is irrelevant):
	\freq, Plprand(60,700,inf),
	\cf, Plprand(800,5000,inf),
	\freqDev, Pwhite(1.01,1.06,inf),
	\rel, 0.14, //A value such as this is equivalent to Pseq([0.14],inf)
	\pan, Pwhite(-1.0,1.0,inf),
	\amp, Plprand(0.1,0.5,inf),
);
)

//Here's the same code, without comments, for the sake of clarity:
(
p = Pbind(
	\type, 'note',
	\instrument, 'sawTone',
	\dur, Plprand(0.05, 0.8, inf),
	\freq, Plprand(60,700,inf),
	\cf, Plprand(800,5000,inf),
	\freqDev, Pwhite(1.01,1.06,inf),
	\rel, 0.14,
	\pan, Pwhite(-1.0,1.0,inf),
	\amp, Plprand(0.1,0.5,inf),
);
)

//Earlier in the semester, we created UGen functions and used the "play" method to create Synths (i.e. {Function}.play creates an instance of a Synth. Recall that a function cannot be "freed", but a Synth can be "freed". Likewise, a Pbind cannot be "stopped", but an EventStreamPlayer can be "stopped".

//p.play returns an EventStreamPlayer, which we'll store using the variable "x".
x = p.play;

//we can now stop "x":
x.stop;

//This, on the other hand, will not work. You'll have to use [command]+[period] to stop the EventStreamPlayer, since we did not store it using a variable:

p.play;

p.stop; //no effect.

//Please experiment with the Pbind above by changing the patterns being used to generate argument values! You could even try making your own SynthDef and your own Pbinds.


//Pbind has a significant number of keys (symbols) that it intrinsically understands and expects, such as freq, amp, out, and others. If you intend to use Pbind, you should name your SynthDef arguments in accordance with Pbind's expected symbolic names, as it allows you to exploit the numerous ways different arguments can be specified. Consider the following two SynthDefs. The first uses the same argument names that Pbind expects (freq and amp), while the latter uses hz and vol.

(
SynthDef(\good, {
	arg freq=440, amp=0.6;
	var env, sig;
	env = EnvGen.kr(Env.perc(0.01,0.5), doneAction:2);
	sig = SinOsc.ar(freq, 0, amp);
	sig = sig * env;
	Out.ar(0, sig!2);
}).add;

SynthDef(\bad, {
	arg hz=440, vol=0.6;
	var env, sig;
	env = EnvGen.kr(Env.perc(0.01,0.5), doneAction:2);
	sig = SinOsc.ar(hz, 0, vol);
	sig = sig * env;
	Out.ar(0, sig!2);
}).add;
)

//as SynthDefs, these are both fine:
Synth(\good);

Synth(\bad);

//Here's the good SynthDef, with expected arguments,
(
x = Pbind(
	\type, \note,
	\instrument, \good,
	\dur, Prand([0.2,0.4],inf),
	\freq, Prand([220,330,440,550],inf),
	\amp, Plprand(0.1,0.5,inf),
).play;
)

x.stop;

//and here's the bad SynthDef, with its expected arguments. Still no problems.
(
x = Pbind(
	\type, \note,
	\instrument, \bad,
	\dur, Prand([0.2,0.4],inf),
	\hz, Prand([220,330,440,550],inf),
	\vol, Plprand(0.1,0.5,inf),
).play;
)

x.stop;

//Pbind intrinsically understands 'freq' and 'amp', so, using our good SynthDef, we can specify these values using alternate symbols. For instance, we can specify amplitude as 'db' instead:
(
x = Pbind(
	\type, \note,
	\instrument, \good,
	\dur, Prand([0.2,0.4],inf),
	\freq, Prand([220,330,440,550],inf),
	\db, Pwhite(-30.0,-6.0,inf), //db instead of amp
).play;
)

x.stop;

//We could specify the frequency using 'midinote':
(
x = Pbind(
	\type, \note,
	\instrument, \good,
	\dur, Prand([0.2,0.4],inf),
	\midinote, Prand([57,64,69,73],inf), //midinote instead of freq
	\db, Pwhite(-30.0,-6.0,inf),
).play;
)

x.stop;

//We could specify frequency using a combination of 'note' and 'gtranspose'. Here, note 0 refers to middle C, and gtranspose shifts the pitches by a number of semitones.
(
x = Pbind(
	\type, \note,
	\instrument, \good,
	\dur, Prand([0.2,0.4],inf),
	\note, Prand([0,7,12,16],inf), //note & gtranspose instead of freq
	\gtranspose, -3,
	\db, Pwhite(-30.0,-6.0,inf),
).play;
)

x.stop;

//We can't take advantage of these options with our bad SynthDef. In the following example, Pbind will calculate a frequency from midinote and an amplitude from db, and use these values in its 'freq' symbol and 'amp' symbol. But, our bad SynthDef doesn't use 'freq', it uses 'hz'. The same problem exists for 'vol' instead of 'amp'. As a result, nothing is controlling this Synth's 'hz' and 'vol' arguments, so it's using the default values (440 and 0.6, respectively).
(
x = Pbind(
	\type, \note,
	\instrument, \bad,
	\dur, Prand([0.2,0.4],inf),
	\midinote, Prand([57,64,69,73],inf),
	\db, Pwhite(-30.0,-6.0,inf),
).play;
)

x.stop;

//See the Pbind help file for more information.
