//More Patterns: A Continuation of Pbind & Associated Concepts

s.reboot;

//Add a SynthDef to be used in subsequent examples:
(
SynthDef(\sawTone, {
	arg	freq=440, freqDev=1,
		widthMin=0.1, widthMax=0.35,
		useFilt=1, cf=5000, amp=0.6,
		atk=0.001, rel=0.4, curve=(-3), pan=0, out=0;
	var env, sig;
	env = EnvGen.kr(Env.perc(atk,rel,1,curve), doneAction:2);
	sig = VarSaw.ar(
		{freq * rrand(freqDev.reciprocal, freqDev)}!8,
		{Rand(0.0,1.0)}!8,
		{ExpRand(widthMin,widthMax)}!8
	);
	sig = Select.ar(useFilt, [sig, LPF.ar(sig, cf)]);
	sig = Mix(sig);
	sig = sig * env;
	sig = Pan2.ar(sig, pan, amp);
	Out.ar(out, sig);
}).add;
)

//Test the SynthDef:
Synth(\sawTone);




//Patterns within Patterns (Patternception)

//Prand will choose from a collection of finite Pseq patterns. Each is exactly one "measure" long. Don't forget that "trace" is a method that allows you to see a pattern's values in the post window.
(
p = Pbind(
	\instrument, \sawTone,
	\dur, Prand([
		Pseq([1],8),
		Pseq([3,0.5,0.5,1,1,2],1),
		Pseq([2,2,1,3],1),
		Pseq([4],2),
		Pseq([8],1)
	],inf).trace,
	\stretch, 0.15,
	\freq, Prand((1..10)*50,inf),
	\rel, 0.18,
);
)

x = p.play;
x.stop;

//Pwrand will choose from one of two internal patterns. The first pattern is a Plprand which will play 16 heavily lowpass filtered notes. The second is a Pseq containing two Pgeom patterns. These Pgeom patterns will ramp up and ramp down the cutoff frequency for a total of 32 notes. Pwrand is more likely to choose the first finite internal pattern (Plprand).
(
p = Pbind(
	\instrument, \sawTone,
	\dur, 0.09,
	\atk, 0.02,
	\rel, 0.12,
	\freq, Prand([40,80],inf),
	\useFilt, 1,
	\cf, Pwrand([
		Plprand(210,400,16),
		Pseq([
			Pgeom(200, 1.34, 16),
			Pgeom(16128, 0.75, 16)
		], 1)
	], [0.7,0.3], inf),
);
)

x = p.play;
x.stop;




//Rests

//It is often difficult to mentally translate delta times into note divisions and meter - for instance, if there is no note on a "downbeat" of an envisioned rhythmic pattern.
(
p = Pbind(
	\instrument, \sawTone,
	\dur, Pseq([1,2,3,3,1,1,2,1,1,1],inf),
	\stretch, 0.15,
	\note, Pseq([16,16,12,14,-2,2,5,10,14,17],inf),
	\db, -15,
	\gtranspose, -7,
	\atk, 0.005,
	\rel, 0.1,
	\useFilt, 0,
	\out, 0
);
)

x = p.play;
x.stop;

//Pbind will interpret a symbol in any frequency stream (midinote, note, degree, etc) as a rest. Here is an alternate way to create the same sounds as the Pbind immediately above. An "empty" symbol (i.e. a single backslash) works perfectly well as a rest symbol.
(
p = Pbind(
	\instrument, \sawTone,
	\dur, 1,
	\stretch, 0.15,
	\note, Pseq([16,16,\,12,\,\,14,\,\,-2,2,5,\,10,14,17],inf),
	\db, -15,
	\gtranspose, -7,
	\atk, 0.005,
	\rel, 0.1,
	\useFilt, 0,
	\out, 0
);
)

x = p.play;
x.stop;




//Linking multiple Pbind Keys

//Here's a simplified Pbind from earlier in this document.
(
p = Pbind(
	\instrument, \sawTone,
	\dur, Prand([
		Pseq([1],8),
		Pseq([3,0.5,0.5,1,1,2],1),
		Pseq([8],1)
	],inf),
	\stretch, 0.15,
	\freq, Prand((1..10)*50,inf),
	\rel, 0.18,
);
)

x = p.play;
x.stop;

//Suppose we want to create a musical association between delta times and frequency. In other words, we want a specific rhythmic pattern to produce a specific pattern of pitches. We can do this with the following syntax, combined with the Ptuple pattern:
(
p = Pbind(
	\instrument, \sawTone,

	//associate the desired keys using a literal array.
	#[dur,freq], Prand([

		//this Ptuple plays eight eighth notes, ascending through the harmonic series.
		Ptuple([
			Pseq([1],8),
			Pseq((1..8)*50,1)
		],1),

		//this ptuple plays a more complex rhythm with random pitches.
		Ptuple([
			Pseq([3,0.5,0.5,1,1,2],1),
			Pwhite(50.0,500.0,6)
		],1),

		//this Ptuple plays a whole note at 2kHz.
		Ptuple([
			Pseq([8],1),
			Pseq([2000],1)
		],1)
	],inf).trace,

	//the Prand immediately above will randomly pick one of three finite Ptuples. Ptuple will return an array of corresponding items produced by its internal patterns. We'll trace this pattern so you can see the results in the post window.

	\stretch, 0.15,
	\rel, 0.18,
);
)

x =p.play;
x.stop;




//Pbind vs Pmono

//Pbind generates a new Synth with every note Event it schedules. Pmono, on the other hand, generates a single Synth and updates its arguments every time a new Event is scheduled.

//Here are two subtly different SynthDefs. The first uses an asr envelope and can therefore sustain indefinitely. Therefore it is suitable for use in a Pmono. The second uses a finite, non-sustainable envelope with doneAction:2, and is therefore better suited for Pbind.
(
SynthDef(\filtSawMono, {
	arg		freq=440, detune=2.0,
			atk=10, rel=10,
			minCf=30, maxCf=6000, minRq=0.005, maxRq=0.04,
			lowShelf=220, rs=0.85, db=6,
			gate=1, amp=0.5, out=0;
	var sig, env;
	env = EnvGen.kr(
		Env.asr(atk,1,rel,-1),
		gate,
		doneAction:2
	);
	sig = Saw.ar(
		freq +
		LFNoise1.kr({LFNoise1.kr(0.5).range(0.15,0.4)}!8).range(detune.neg,detune)
	);
	sig = BPF.ar(
		sig,
		LFNoise1.kr({LFNoise1.kr(0.13).exprange(0.02,0.25)}!8).exprange(minCf, maxCf),
		LFNoise1.kr({LFNoise1.kr(0.08).exprange(0.08,0.35)}!8).range(minRq, maxRq)
	);
	sig = BLowShelf.ar(sig, lowShelf, rs, db);
	sig = Splay.ar(sig);
	sig = sig * env * 10 * amp;
	sig = Limiter.ar(sig, 0.95);
	Out.ar(out, sig);
}).add;

SynthDef(\filtSaw, {
	arg		freq=440, detune=2.0, atk=6, sus=4, rel=6,
			minCf=30, maxCf=6000, minRq=0.005, maxRq=0.04,
			lowShelf=220, rs=0.85, db=6,
			amp=0.5, out=0;
	var sig, env;
	env = EnvGen.kr(
		Env.new(
			[0,1,1,0],
			[atk,sus,rel],
			[1,0,-1]
		),
		doneAction:2
	);
	sig = Saw.ar(
		freq +
		LFNoise1.kr({LFNoise1.kr(0.5).range(0.15,0.4)}!8).range(detune.neg,detune));
	sig = BPF.ar(
		sig,
		LFNoise1.kr({LFNoise1.kr(0.13).exprange(0.02,0.25)}!8).exprange(minCf, maxCf),
		LFNoise1.kr({LFNoise1.kr(0.08).exprange(0.08,0.35)}!8).range(minRq, maxRq)
	);
	sig = BLowShelf.ar(sig, lowShelf, rs, db);
	sig = Splay.ar(sig);
	sig = sig * env * 10 * amp;
	sig = Limiter.ar(sig, 0.95);
	Out.ar(out, sig);
}).add;
)

//test the first Synth
x = Synth(\filtSawMono, [\freq, 60])

//manually free the Synth
x.set(\gate, 0)

//this Synth will free itself via doneAction:2
x = Synth(\filtSaw, [\freq, 60])

//This Pbind will generate a new Synth every 1 to 4 seconds with a random frequency. Check the node tree to see for yourself.

s.makeGui;

(
p = Pbind(
	\instrument, \filtSaw,
	\dur, Pwhite(1.0,4.0,inf),
	\freq, Plprand(40,250,inf),
	\atk, 5,
	\rel, 5,
	\amp, 0.3,
);
)

x = p.play;
x.stop;

//Pmono, on the other hand, will generate a single Synth and update its arguments every 1 to 4 seconds
(
p = Pmono(
	\filtSawMono,
	\dur, Pwhite(1.0,4.0,inf),
	\freq, Plprand(40,250,inf),
	\amp, 0.5,
);
)

x = p.play;
x.stop;

//It is possible to glissando from one pitch to the next with a cleverly designed SynthDef:
(
SynthDef(\filtSawGliss, {
	arg		freq=440, detune=2.0,
			atk=10, rel=10,
			minCf=30, maxCf=6000, minRq=0.005, maxRq=0.04,
			lowShelf=220, rs=0.85, db=6,
			t_trig=1, glissTime=0.3,
			gate=1, amp=0.5, out=0;
	var sig, env, freqEnv;

	env = EnvGen.kr(
		Env.asr(atk,1,rel,-1),
		gate,
		doneAction:2
	);

	//our freqency is now being controlled by a retriggerable envelope, normally sitting at a value of 1, but in this case being "levelScaled" by our frequency argument, re-triggered by setting t_trig equal to 1
	freqEnv = EnvGen.kr(Env.new([1,1],[0.001],\lin), t_trig, levelScale:freq);

	//we then "lag" the frequency signal by a lag time, specified by the argument "glissTime". When Lag's input signal changes, it takes glissTime seconds to actually reach this new values. So when we retrigger our freqEnv with a new levelScale value, the pitch will glissando over the time amount we specify.
	freqEnv = Lag.kr(freqEnv, glissTime);

	sig = Saw.ar(
		freqEnv +
		LFNoise1.kr({LFNoise1.kr(0.5).range(0.15,0.4)}!8).range(detune.neg,detune)
	);
	sig = BPF.ar(
		sig,
		LFNoise1.kr({LFNoise1.kr(0.13).exprange(0.02,0.25)}!8).exprange(minCf, maxCf),
		LFNoise1.kr({LFNoise1.kr(0.08).exprange(0.08,0.35)}!8).range(minRq, maxRq)
	);
	sig = BLowShelf.ar(sig, lowShelf, rs, db);
	sig = Splay.ar(sig);
	sig = sig * env * 10 * amp;
	sig = Limiter.ar(sig, 0.95);
	Out.ar(out, sig);
}).add;
)

//This Pmono updates the Synth's arguments every 1.1 seconds, each time choosing a random frequency and glissTime. It is also necessary to update the t_trig argument to 1 in order to re-trigger the frequency envelope.
(
p = Pmono(
	\filtSawGliss,
	\t_trig, 1,
	\dur, 1.1,
	\freq, Plprand(5,150,inf),
	\glissTime, Pwhite(0.1,1.0,inf),
	\amp, 0.5,
);
)

y = p.play;
y.stop;





//Pattern Proxies: Real-time Pattern Manipulation

//Wrapping any value pattern in a Pdefn enables you to refer to this pattern by a symbolic name and change it as the EventStreamPlayer is running.
(
p = Pbind(
	\instrument, \filtSaw,
	\dur, Pdefn(\myDur, Pwhite(1.0,5.0,inf)),
	\freq, Pdefn(\myFreq, Plprand(40,250,inf)),
	\detune, Pwhite(1.0, 10.0, inf),
	\atk, Pdefn(\myAtk, Pkey(\dur) * 0.5),
	\sus, Pdefn(\mySus, Pkey(\dur) * 0.5),
	\rel, Pdefn(\myRel, Pkey(\dur) * 0.5),
);
)

x = p.play;

//we'll redefine the freq Pattern to only play sub-audio frequencies. This will take effect as soon as the next note Event is generated.
Pdefn(\myFreq, Plprand(2,17,inf));

//Re-define these Pdefn Patterns simultaneously to shorten the delta time as well as the argument duration patterns.
(
Pdefn(\myDur, Plprand(0.1,0.2,inf));
Pdefn(\myAtk, Pwhite(0.01,0.2,inf));
Pdefn(\mySus, Phprand(0.02,0.5,inf));
Pdefn(\myRel, Pwhite(0.01,0.2,inf));
)

//Return to the original settings.
(
	Pdefn(\myDur, Pwhite(1.0,5.0,inf));
	Pdefn(\myFreq, Plprand(40,250,inf));
	Pdefn(\myAtk, Pkey(\dur) * 0.5);
	Pdefn(\mySus, Pkey(\dur) * 0.5);
	Pdefn(\myRel, Pkey(\dur) * 0.5);
)

x.stop;

//The possibilities are endless!
