//DiskIn and DiskOut are UGens that allow you to read audio from your hard drive and write audio to a file on your hard drive, respectively. In the first section of code below, I have three SynthDefs: a sound source, a SynthDef that writes a file via DiskOut, and a SynthDef that reads a file via DiskIn.

//See the very bottom of this document for a much easier and convenient way to real-time record audio to disk.

s.makeGui;

(
//sound source
SynthDef(\bubbly, {
	arg	freq=20, detune=1, widthMin=0.03, widthMax=0.35,
		cfMin=100, cfMax=8000, cfRound=1, rqMin=0.005, rqMax=0.11,
		atk=3, rel=3, gate=1, amp=1;
	var sig, freqDev, widthCtrl, bpfCtrl, rqCtrl, env;

	//amplitude envelope
	env = EnvGen.kr(Env.asr(atk,1,rel,-1), gate, doneAction:2);

	//frequency detuning control signal (8ch)
	freqDev = LFNoise1.kr({ExpRand(0.01,0.2)}!8).range(detune.neg, detune);

	//center frequency control for bandpass filter (8ch)
	bpfCtrl = LFNoise1.kr({Rand(0.02,0.1)}!8)
		.exprange(cfMin, cfMax)
		.round(cfRound)
		.max(cfRound);

	//control for filter quality reciprocal (8ch)
	rqCtrl = LFNoise1.kr({Rand(0.02,0.2)}!8)
		.exprange(rqMin, rqMax);

	//audio signal (8ch)
	sig = Pulse.ar(freq * freqDev);

	//apply bandpass filter (8ch)
	sig = BPF.ar(
		sig,
		bpfCtrl,
		rqCtrl
	);

	//Splay 8ch signal to 2ch field
	sig = Splay.ar(sig);

	//amplitude envelope and other amplitude adjustments
	sig = sig * env;
	sig = sig * 10;
	sig = Limiter.ar(sig, 0.99);
	sig = sig * amp;

	Out.ar(0, sig);
}).add;

//record to disk
SynthDef(\diskRec, {
	arg buf;
	var recSig;
	recSig = In.ar(0,2);
	DiskOut.ar(buf, recSig);
}).add;



//read from disk
SynthDef(\diskPlay, {
	arg buf;
	var sig;
	sig = DiskIn.ar(2, buf, 0);
	Out.ar(0, sig);
}).add;
)

//if we want to write an audio file to disk, we first need to allocate a buffer in SC. When using DiskIn/DiskOut, the number of frames in the buffer must be a power of 2. This is a stereo buffer on the local server with 262144 frames (about 6 seconds long).
b = Buffer.alloc(s, 2.pow(18), 2);

//IMPORTANT: In the code below, you will need to change the full file path to a path which is valid on your own computer.

//"write" is a Buffer instance method that creates an audio file on your hard drive to which the contents of the buffer will be written.
(
b.write(

	//the full path of the new file
	"/Users/eli/Desktop/diskOutTest.aiff",

	//the file type (header type)
	"aiff",

	//the sample format (see the help file for SoundFile for more info on header types and sample formats)
	"int16",

	//The number of samples to write. currently, our Buffer b is empty (full of zeroes) so we don't to write this to the hard disk just yet. So we'll write zero of these samples to disk.
	0,

	//The frame in the buffer at which to start writing audio data to disk. We'd like to start at the beginning.
	0,

	//Whether we want to leave the file "open" for recording purposes. "true" indicates that we're not done recording data to the hard disk. "false" completes the file creation process and finalizes the audio file.
	true
);
)

//instantiate the sound source but initialize the gate at zero.
~soundSource = Synth(\bubbly, [\gate, 0]);


//"makeBundle" allows several server commands to be executed with sample-precise synchronicity. This method expects a time to wait before executing the commands, and a function containing said commands. In this case, we want to instantiate the DiskOut Synth and set the sound source's gate argument to be equal to one. The 1-second delay time is "big enough," and somewhat arbitrary.
(
s.makeBundle(1, {

	//instantiate the record synth (AFTER the sound source!)
	~diskRec = Synth(\diskRec, [\buf, b.bufnum], ~soundSource, \addAfter);

	//trigger the soundsource
	~soundSource.set(\gate, 1);
});
)

//Run this line to fade and free the sound source.
~soundSource.set(\gate, 0);

//When the sound has completely faded out, free the record synth, which will terminate disk writing
~diskRec.free;

//"close" lets SC know that we're done recording and it can complete the aiff-creation process.
b.close;

//After closing the buffer, you should be able to find this file in your operating system and play it through just about any audio application (Peak, Audacity, Logic, ProTools, iTunes, etc). Although we weren't met with success during class, I tried the exact same code after class and had no problems.

//We now have an audio file loaded in memory as Buffer b, and an aiff file on disk (with whatever name & path you've given it). We can optionally play back the file we've recorded using our DiskIn SynthDef:
~diskPlay = Synth(\diskPlay, [\buf, b.bufnum]);

//free the DiskIn Synth
~diskPlay.free;

//Or, we could use it with PlayBuf and/or BufRd:
x = {BufRd.ar(2,b,SinOsc.ar(0.08,3pi/2).range(0,BufFrames.kr(b)/5))}.play;
x.free;

x = {PlayBuf.ar(2, b, 0.25, doneAction:2)}.play;
x.free;

//When we're absolutely done, free the buffer so SC can reallocate the memory space.
b.free;


//All of the above is the formal, sophisticated way to record audio to disk. There's a much easier way:

//On the Server GUI,

s.makeGui;

//You'll notice there is a button labeled "record." You see where I'm going with this. By default, pressing this button will write SC's stereo output on hardware busses 0 and 1 to an audio file on your hard disk in real-time. Press it again to stop recording. Check the post window for information on where the file has been written to and what it has been named.

//If you are working with 4 or 8 channel sound, you can change the number of busses that are recorded to disk:
s.recChannels = 4;

//After rebooting the server,
s.reboot;

//SC will record this many busses, starting with bus 0.
