//Ppar and Pseq: Event Patterns other than Pbind and Pmono

s.makeGui.boot;

(
SynthDef(\percSaw, {
	arg freq=440, amp=1, atk=0.01, rel=0.1, cf=4000, pan=0, out=0;
	var sig, env;
	env = EnvGen.kr(Env.perc(atk,rel), doneAction:2);
	sig = Saw.ar(freq);
	sig = LPF.ar(sig, cf);
	sig = sig * env;
	sig = Pan2.ar(sig, pan, amp);
	Out.ar(out, sig);
}).add;

~scale1 = Pbind(
	\instrument, \percSaw,
	\dur, 0.2,
	\amp, 0.3,
	\midinote, Pseq([60,62,64,65,67,69,71,72],1),
	\cf, 3000,
	\pan, 0,
	\atk, 0.01,
	\rel, 0.1,
	\out, 0,
);

~scale2 = Pbind(
	\instrument, \percSaw,
	\dur, 0.2,
	\amp, 0.3,
	\midinote, Pseq([60,62,64,65,67,69,71,72],1),
	\ctranspose, 3,
	\cf, 3000,
	\pan, 0,
	\atk, 0.01,
	\rel, 0.1,
	\out, 0,
);
)

//Immediately above, we have two slightly different Pbinds. The resulting EventStreamPlayers produce one iteration of a major scale starting on different notes:

~scale1.play;

~scale2.play;

//In many cases, patterns can be played simultaneously simply by evaluating multiple lines of code simultaneously, with no audible delay time between them:

(
~scale1.play;
~scale2.play;
)

//However, it's slightly better and safer to use Ppar for playing event patterns simultaneously. Ppar is a pattern which takes an Array of multiple event streams (i.e. Pbinds and Pmonos) and produces a single event stream by playing the event patterns in parallel (hence the name). Ppar, like Pbind and Pmono, can be "played", thereby producing an EventStreamPlayer.

x = Ppar([~scale1, ~scale2], 1).play; //finite

x = Ppar([~scale1, ~scale2], inf).play; //infinite
x.stop;

//Suppose we wanted to sequence these patterns instead of playing them simultaneously. Although we've mostly been using Pseq to generate numerical values, Pseq can handle event patterns too (i.e. Pseq can also be "played" if its contents happen to be event patterns):

x = Pseq([~scale1, ~scale2], 1).play; //finite

x = Pseq([~scale1, ~scale2], inf).play; //infinite
x.stop;

//In class today, the question was posed: "What if we wanted to start with the first pattern, and then have the second pattern join on the next iteration?

//Here's how we'd accomplish this. Conceptually, we're dealing with a sequential event pattern that has two components. First, we want to play ~scale1 once by itself. Then, we want to play ~scale1 and ~scale2 in parallel, infinitely. So we'd put two items inside a Pseq -- the first is simply ~scale1, and the second is a Ppar containing both ~scale1 and ~scale2:

(
x = Pseq(
	[
	~scale1,
	Ppar(
		[
		~scale1,
		~scale2
		], inf)
	], 1).play;
)

//Often, creating more complex event patterns (and ultimately, a piece of music) from smaller event patterns is simply a matter of creating the appropriate construction of nested and/or interconnected Ppars and Pseqs. It's not hard to imagine a structure like this:

(
//basic instrumental components
~melody1 = Pbind(...);
~melody2 = Pbind(...);
~melody3 = Pbind(...);
~drums1 = Pbind(...);
~drums2 = Pbind(...);
~drums3 = Pbind(...);
~bass1 = Pbind(...);
~bass2 = Pbind(...);
~bass3 = Pbind(...);

//larger sections
~intro = Ppar([~drums1, ~bass1], 1); //the intro is just drums & bass
~verse1 = Ppar([~melody1, ~drums1, ~bass1], 1);
~verse2 = Ppar([~melody2, ~drums2, ~bass2], 1);
~chorus = Ppar([~melody3, ~drums3, ~bass3], 1);
~outro = Ppar([~melody3, ~drums3], 1); //the outro is just melody and drums

//whole piece
~myPiece = Pseq([
	~intro,
	~verse1,
	~chorus,
	~verse2,
	Pseq([~chorus], 2),
	~outro
], 1);
)

//play the piece
~myPiece.play;
