//An Observation on Pshuf:

//I had been under the impression that Pshuf would randomize the order of a collection on EACH repeat. However, this is not the case. Pshuf will randomize the order of a collection and then PRESERVE this random ordering for subsequent repeats. For example:

Pshuf([ 1, 2, 3, 4, 5 ], 3).asStream.all;

//Notice that the initially chosen random order is preserved for all three repeats.

//If you'd like to produce a stream of values with a new random ordering on EACH repeat, this can be achieved using a combination of Pshuf and Pseq:

Pseq([ Pshuf([ 1, 2, 3, 4, 5 ], 1) ], 3).asStream.all;

//In the previous example, Pshuf chooses a random order and repeats this order for a total of three iterations. In contrast to the previous example, it is now Pseq that handles the threefold repetition of the value stream, instead of Pshuf. Pseq will produce three unique iterations of Pshuf, and therefore choose a new random ordering on each repeat.



//Introducing: Pseg

//Pseg is a value pattern that behaves in the same way as the breakpoint (or "segmented") envelope generated by Env.new. Values controlled by a Pseg will change according to absolute time values, rather than the relative delta times generated by a rhythmic duration pattern.

//Add this SynthDef first
(
SynthDef(\short, {
	arg freq=440, amp=1, atk=0.01, rel=0.1, cf=4000, pan=0, out=0;
	var sig, env;
	env = EnvGen.kr(Env.perc(atk,rel), doneAction:2);
	sig = Saw.ar(freq);
	sig = LPF.ar(sig, cf);
	sig = sig * env;
	sig = Pan2.ar(sig, pan, amp);
	Out.ar(out, sig);
}).add;
)

//Pseg, like Env.new, requires three arguments: an Array of levels, an Array of times, and an Array of curves. When using Pseg, these three Arrays are usually enclosed within three Pseq patterns. For comparison:

/*

Env.new(levels, times, curves);

Pseg( Pseq(levels, repeats), Pseq(times, repeats), Pseq(curves, repeats) );

*/

//In the following Pbind, we are using a Pseg to control the amplitude values. According to the first internal Pseq, the amplitude will travel from 0 to 1 and back to 0. According to the times Array in the second Pseq, the pattern will ramp up to full amplitude over 5 seconds, then back to silence over another 5 seconds. The final Pseq specifies the curves of these interpolations.

//Just like Env.new, the first Array has one more item in it than the other two Arrays (i.e. we have three level points, but there are only two pairs of points to interpolate between).

//Because the Pseq patterns only undergo one iteration, they are finite, and therefore the resulting EventStreamPlayer will also be finite; the EventStreamPlayer will terminate after 10 seconds.

(
p = Pbind(
	\instrument, \short,
	\dur, 0.6,
	\amp, Pseg(Pseq([0,1,0],1),Pseq([5,5],1),Pseq([1,-1],1)),
	\freq, Pxrand([55,110,220,440],inf),
	\cf, Plprand(60,6000,inf),
	\pan, Pwhite(-0.5,0.5,inf),
).play;
)

//To create an infinite EventStreamPlayer incorporating Pseg, we only need to do a couple of things:

//First, all three internal Arrays (levels, times, curves) must be the same size. The reason: we'll need an additional time & curve value to return from the last level point to the first level point. In this case, the simplest solution is to shave off the last item in the levels Array.

//Then, we need to specify that the internal Pseq patterns repeat indefinitely.

//Now, our EventStreamPlayer will ramp from 0 to 1 over 5 seconds, then from 1 back to 0 over 5 seconds, and will repeat this pattern indefinitely.

(
x = Pbind(
	\instrument, \short,
	\dur, 0.1,
	\amp, Pseg(Pseq([0.001,1],inf),3,\exp),
	\freq, Pxrand([55,110,220,440],inf),
	\cf, Plprand(60,6000,inf),
	\pan, Pwhite(-0.5,0.5,inf),
).play;
)

x.stop;

//Here's a neat trick that involves Pdefn and Pseg that can be used to fade in and fade out an EventStreamPlayer. Here's a slightly modified version of the example immediately above.

//Our amplitude values are now wrapped in a Pdefn, so we can change the EventStreamPlayer in real-time. Inside this Pdefn is a Pseq, with two items: the first is a finite Pseg which ramps from 0 to 0.7 over 5 seconds. After this Pseg ends, Pseq sequentially steps to its next item, which is an infinite Plprand, ranging from 0.5 to 0.8. Since this Plprand is infinite, the outermost Pseq does not advance beyond this internal pattern.

//I'm using "trace", so watch the post window for visual confirmation.

(
x = Pbind(
	\instrument, \short,
	\dur, 0.1,
	\amp, Pdefn(
		\shortAmp,
		Pseq([
			Pseg(Pseq([0,0.7],1),Pseq([5],1),Pseq([1],1)).trace,
			Plprand(0.5,0.8,inf).trace
		], 1)
	),
	\freq, Pxrand([55,110,220,440],inf),
	\cf, Plprand(60,6000,inf),
	\pan, Pwhite(-0.5,0.5,inf),
).play;
)

//Now, all we have to do is replace the current Pdefn with a new Pdefn containing a Pseg that serves to fade out the amplitude values:
Pdefn(\shortAmp, Pseg(Pseq([0.7,0],1),Pseq([5],1),Pseq([-1],1)).trace);


//Here are a few more examples in which a Pseg controls other parameters:

//Pseg controls cutoff frequency - 600Hz to 6kHz to 60Hz, 3-second interpolations, repeating indefinitely:
(
x = Pbind(
	\instrument, \short,
	\dur, Prand([0.14,0.07],inf),
	\amp, Plprand(0.05,0.7,inf),
	\midinote, Pxrand([34,46,58,60,65,69],inf),
	\cf, Pseg(Pseq([600,6000,60],inf),Pseq([3,3,3],inf),Pseq([1,-1,1],inf)),
	\pan, Pwhite(-0.5,0.5,inf),
	\rel, Pwrand([Pwhite(0.08,0.12,1), Pwhite(2.0,3.0,1)],[0.945,0.055],inf),
).play
)

x.stop;

(
x = Pbind(
	\instrument, \short,
	\dur, Prand([0.14,0.07],inf),
	\amp, Plprand(0.05,0.7,inf),
	\midinote, Pxrand([34,46,58,60,65,69],inf),
	\cf, Pseg(Pseq([60,6000],inf), Pseq([3,3],inf), \exp),
	\pan, Pwhite(-0.5,0.5,inf),
	\rel, Pkey(\cf).expexp(60,6000,0.05,2.0),
).play
)


//Pseg controls pan position
(
x = Pbind(
	\instrument, \short,
	\dur, Prand([0.14,0.07,0.21],inf),
	\amp, Plprand(0.05,0.7,inf),
	\midinote, Pxrand([39,41,46,48,53,55,60],inf),
	\cf, Plprand(60,6000,inf),
	\pan, Pseg(Pseq([0,-0.9,0.9,0],inf),Pseq([2,4,4,2],inf),\lin),
	\rel, Pwhite(0.04,0.08,inf),
).play
)

x.stop;

//Pseg controls frequency
(
x = Pbind(
	\instrument, \short,
	\dur, Plprand(0.05,0.3,inf),
	\amp, Plprand(0.02,0.25,inf),
	\freq, Pseg(Pseq([40,600],inf),Pseq([2,2],inf),Pseq([1,-1],inf)),
	\cf, Plprand(60,6000,inf),
	\pan, Pwhite(-0.6,0.6,inf),
	\atk, Pwhite(0.01,0.3,inf),
	\rel, Phprand(0.2,1.8,inf),
).play
)

x.stop;

//Same as above, but notice that the "round" method is valid for numerical patterns:
(
x = Pbind(
	\instrument, \short,
	\dur, Plprand(0.05,0.3,inf),
	\amp, Plprand(0.02,0.25,inf),
	\freq, (Pseg(Pseq([40,600],inf),Pseq([2,2],inf),Pseq([1,-1],inf)).round(Pseq([40,50],inf))).trace,
	\cf, Plprand(60,6000,inf),
	\pan, Pwhite(-0.6,0.6,inf),
	\atk, Pwhite(0.01,0.3,inf),
	\rel, Phprand(0.2,1.8,inf),
).play
)

x.stop;

Ppar


(
x = Pbind(
	\instrument, \short,
	\dur, 0.2,
	\amp, 0.3,
	\midinote, Pseq([60,62,64,65,67,69,71,72],1),
	\cf, 3000,
	\pan, 0,
	\atk, 0.01,
	\rel, 0.1,
);
y = Pbind(
	\instrument, \short,
	\dur, 0.2,
	\amp, 0.3,
	\midinote, Pseq([60,62,64,65,67,69,71,72],1),
	\ctranspose, 3,
	\cf, 3000,
	\pan, 0,
	\atk, 0.01,
	\rel, 0.1,
)
)

z = Pseq(
	[x],
	Pseq(
		[Ppar([x,y])],
		inf
	)
).play
z.stop



//And of course, it's perfectly reasonable to use Pseg for multiple arguments within a single Pbind.