//Granular Synthesis with Pbind

//Supply your own buffer here by replacing the string with a valid path name
b = Buffer.readChannel(s, "/Users/eli/Desktop/whateverHappened.aiff", 0, -1, 0);

//Test the buffer to make sure it has been loaded onto the server
b.play;

//Pbind generates a stream of events. In the context of note events, we only need to define a SynthDef that generates a single grain. Using appropriate arguments, Pbind becomes a very convenient granular synthesizer.

//I'm using Env.sine, so the grain envelope shape is not very flexible. However, you can replace Env.sine with Env.new and provide additional arguments in order to add flexibility. You could also place multiple EnvGen.kr's inside a Select.kr in order to select from several different envelope types.
(
SynthDef(\grain, {
	arg gdur=0.02, buf=0, rate=1, spos=0, pan=0, amp=0.5, out=0;
	var sig, env;
	env = EnvGen.kr(Env.sine(gdur), doneAction:2);
	sig = PlayBuf.ar(1, buf, BufRateScale.kr(buf)*rate, startPos:spos, loop:1);
	sig = sig * env;
	sig = Pan2.ar(sig,pan,amp);
	Out.ar(out, sig);
}).add;
)

//No patterns - as simple as it gets
(
x = Pbind(
	\instrument, \grain,
	\dur, 0.1, //constant hop size (synchronous granular synthesis)
	\gdur, 0.03, //constant grain size
	\buf, b.bufnum,
	\rate, 1, //no transposition
	\spos, 1 * s.sampleRate, //static grain pointer
	\pan, 0, //panned to the center
	\amp, 0.5,
	\out, 0,
).play;
)

x.stop;

//Using Pseg to advance the grain pointer through the buffer, preserving the original duration
(
x = Pbind(
	\instrument, \grain,
	\dur, 0.05,
	\gdur, 0.03,
	\buf, b.bufnum,
	\rate, 1,
	\spos, Pseg(Pseq([0,b.numFrames],inf), Pseq([b.duration,0],inf), \lin),
	\pan, 0,
	\amp, 0.5,
	\out, 0,
).play;
)

x.stop;

//Using Pwhite to randomize the grain pointer
(
x = Pbind(
	\instrument, \grain,
	\dur, 0.05,
	\gdur, 0.03,
	\buf, b.bufnum,
	\rate, 1,
	\spos, Pwhite(0, b.numFrames-(Pkey(\gdur)*s.sampleRate), inf),
	\pan, 0,
	\amp, 0.5,
	\out, 0,
).play;
)

x.stop;

//Using Pgauss to randomize hop size (grain density)
(
x = Pbind(
	\instrument, \grain,
	\dur, Pgauss(0.2, 0.1, inf).trace,
	\gdur, 0.03,
	\buf, b.bufnum,
	\rate, 1,
	\spos, Pwhite(0, b.numFrames-(Pkey(\gdur)*s.sampleRate), inf),
	\pan, 0,
	\amp, 0.5,
	\out, 0,
).play;
)

x.stop;

//Using Pshuf to create asynchronous granular synthesis, while preserving an exact number of grains per second
(
x = Pbind(
	\instrument, \grain,

	//Pshuf is given an array of 10 different delta times (hop sizes). It chooses a new ordering of these delta times on each iteration. Therefore, with a multiplier of 1, We get exactly 10 grains per second. If you change the multiplier to 2, we will get 10 grains every 2 seconds, an so on.
	\dur, Pseq([Pshuf((0.01,0.02..0.1).normalizeSum * 1, 1)], inf),
	\gdur, 0.02,
	\buf, b.bufnum,
	\rate, 1,
	\spos, Pwhite(0, b.numFrames-(Pkey(\gdur)*s.sampleRate), inf),
	\pan, 0,
	\amp, 0.5,
	\out, 0,
).play;
)

x.stop;

//Pitch-shifting, independent of time stretching. "midiratio" takes a number of half steps and converts it to a frequency ratio. Here we are transposing by a major third (4 half steps).
(
x = Pbind(
	\instrument, \grain,
	\dur, 0.05,
	\gdur, 0.03,
	\buf, b.bufnum,
	\rate, 4.midiratio,
	\spos, Pseg(Pseq([0,b.numFrames],inf), Pseq([b.duration,0],inf), \lin),
	\pan, 0,
	\amp, 0.5,
	\out, 0,
).play;
)

x.stop;

//Don't forget, providing an array of values will create multiple Pbind streams!
(
x = Pbind(
	\instrument, \grain,
	\dur, 0.05,
	\gdur, 0.03,
	\buf, b.bufnum,
	\rate, [0,4,7].midiratio, //major triad
	\spos, Pseg(Pseq([0,b.numFrames],inf), Pseq([b.duration,0],inf), \lin),
	\pan, 0,
	\amp, 0.5,
	\out, 0,
).play;
)

x.stop;

//Time stretching, independent of pitch-shifting. Change the multiplier of b.duration to stretch/compress the length of time it takes to traverse the buffer.
(
x = Pbind(
	\instrument, \grain,
	\dur, 0.05,
	\gdur, 0.1,
	\buf, b.bufnum,
	\rate, 1,
	\spos, Pseg(Pseq([0,b.numFrames],inf), Pseq([3*b.duration,0],inf), \lin),
	\pan, 0,
	\amp, 0.5,
	\out, 0,
).play;
)

x.stop;

//Remember that ALL parameters are subject to patternization within a Pbind. Experiment!