//Real-time granular synthesis involves writing a live signal to an allocated buffer while simultaneously granulating the buffer. An implementation in SC is best achieved by creating three SynthDefs which each serve a unique purpose: outputting an audio rate index (pointer) into a buffer, recording a signal into a buffer according to pointer, and granulating a buffer according to a lagged pointer.

s.reboot.makeGui;

//Allocate a 4-second buffer. In the case of real-time granular synthesis, we treat this buffer as a so-called "circular buffer", which means that when the pointer reaches the end of the buffer, it will wrap back to the beginning of the buffer.
b = Buffer.alloc(s, s.sampleRate*4, 1);

//Allocate a bus for the audio rate index (pointer)
~ptrBus = Bus.audio(s,1);

//Allocate a bus for the live signal to be granulated
~liveInputBus = Bus.audio(s,1);

(
//A simple SoundIn SynthDef for inputting microphone signal
SynthDef(\liveInput, {
	arg in=0, out, amp=1;
	var sig;
	sig = SoundIn.ar(in);
	Out.ar(out, sig);
}).add;

//Phasor is ideal for a grain pointer
SynthDef(\grainPtr, {
	arg t_reset=1, buf, out;
	var ptr;
	ptr = Phasor.ar(t_reset, 1, 0, BufFrames.kr(buf)-1, 0);
	Out.ar(out, ptr);
}).add;

//This SynthDef will begin recording into a buffer when t_on is triggered, and will stop recording when t_off is triggered. When either of these arguments is triggered, the SynthDef will crossfade between the current buffer contents and the live incoming signal, thereby eliminating the possibility of a discontinuity in the circular buffer's stored audio.
SynthDef(\grainRec, {
	arg buf, ptrBus, inBus, t_on=0, t_off=0, lagTime=1, amp=1;
	var recSig, ptr, bufContents, xfade, sig;

	//incoming live mic signal
	recSig = In.ar(inBus, 1);

	//grain pointer
	ptr = In.ar(ptrBus, 1);

	//current buffer contents, accessed via BufRd
	bufContents = BufRd.ar(1, buf, ptr, 1, 2);

	//the crossfade control signal is a combination of SetResetFF and Lag3
	xfade = SetResetFF.kr(t_on, t_off);
	xfade = xfade.linlin(0,1,-1,1);
	xfade = Lag3.kr(xfade, lagTime);

	//crossfade the current contents with the incoming mic signal
	sig = XFade2.ar(bufContents, recSig, xfade, amp);

	//write this crossfaded signal to the buffer
	BufWr.ar(sig, buf, ptr, 1);

	//no output for this SynthDef
}).add;

//This SynthDef will granulate a buffer, generating grains at locations specified by a grain pointer that lags slightly behind the record pointer. The granular textures are subject to many parameters, which are responsible for the large number of arguments and variables.
SynthDef(\grainGen, {
	arg buf, ptrBus, ptrLagSec=0.3,
		gHzMin=8, gHzMax=20, gHzFreq=10, gGenType=0,
		gDurMin=0.02, gDurMax=0.08, gDurFreq=20,
		gRateMin=1.0, gRateMax=1.0, gRateFreq=20,
		ptrDev=0.1, ptrDevFreq=10,
		pan=0.5, panFreq=10, amp=1, out=0;
	var gDurCtrl, gHzCtrl, gRateCtrl, ptrDevCtrl, panCtrl, gGenCtrl,
		playPtr, ptrLagSmps, ptrDevSmps, sig;

	//convert pointer deviation from seconds to samples
	ptrDevSmps = ptrDev * s.sampleRate;

//I like to use LF noise generators with explicitly specified ranges in order to control the parameters of the granular synthesis, rather than static values. A small or large range can be chosen, or the min and mix values can be set equal to one another to create a static value.

	//grain duration in seconds
	gDurCtrl = LFNoise1.kr(gDurFreq).range(gDurMin, gDurMax);

	//grain frequency/density
	gHzCtrl = LFNoise1.kr(gHzFreq).exprange(gHzMin, gHzMax);

	//grain playback rate (pitch ratio)
	gRateCtrl = LFNoise1.kr(gRateFreq).exprange(gRateMin,gRateMax);

	//pointer deviation in samples
	ptrDevCtrl = LFNoise1.kr(ptrDevFreq).bipolar(ptrDevSmps);

	//pan position
	panCtrl = LFDNoise1.kr(panFreq).range(pan.neg, pan);

	//synchronous vs asynchronous
	gGenCtrl = Select.kr(gGenType, [Dust.kr(gHzCtrl), Impulse.kr(gHzCtrl)]);

//Various granular parameters (such as grain duraion, grain playback rate, and pointer deviation) may create a situation in which a generated grain will extend beyond the record pointer and create a click. Therefore, we compare the pointer lag time to the length of the largest possible grain, and set the actual pointer lag time equal to the maximum of these two values.
	ptrLagSec = max(ptrLagSec, gDurMax + ptrDev);

	//convert adjusted pointer lag time from seconds to samples
	ptrLagSmps = ptrLagSec * s.sampleRate;

	//pointer for grain generation
	playPtr = In.ar(ptrBus);

	//add lag amount
	playPtr = (playPtr - ptrLagSmps);

	//add pointer deviation
	playPtr = playPtr + ptrDevCtrl;

	//wrap pointer to buffer size, in samples
	playPtr = playPtr % BufFrames.kr(buf);

	//convert grain pointer to a range of [0,1] for GrainBuf
	playPtr = playPtr/BufFrames.kr(buf);

	//finally! make the grains
	sig = GrainBuf.ar(2, gGenCtrl, gDurCtrl, buf, gRateCtrl, playPtr, 2, panCtrl, -1);
	Out.ar(out, sig);
}).add;
)

(
~makeNodes = {
	s.makeBundle(nil, {
		~inputGrp = Group.new;
		~genGrp = Group.after(~inputGrp);
		~ptrSynth = Synth.new(\grainPtr, [\buf, b.bufnum, \out, ~ptrBus], ~inputGrp);
		~liveInputSynth = Synth.new(\liveInput, [\out, ~liveInputBus], ~inputGrp);
	});
};
ServerTree.add(~makeNodes);
s.freeAll;
)

(

~grainRecSynth = Synth.new(
	\grainRec,
	[
		\buf, b.bufnum,
		\t_on, 1,
		\ptrBus, ~ptrBus,
		\inBus, ~liveInputBus,
	],
	~genGrp
);

~grainGenSynth = Synth.new(
	\grainGen,
	[
		\buf, b.bufnum,
		\ptrBus, ~ptrBus,
	],
	~genGrp
);

~ptrSynth.set(\t_reset, 1);

)



(
//instantiate record synth & granulation synth simulatenously
~grainRecSynth = Synth.new(
	\grainRec,
	[
		\buf, b.bufnum,
		\t_on, 1,
		\ptrBus, ~ptrBus,
		\inBus, ~liveInputBus
	],
	~genGrp
);

~grainGenSynth1 = Synth.new(
	\grainGen,
	[
		\buf, b.bufnum,
		\ptrBus, ~ptrBus,
		\amp, 0.75,
		\ptrLagSec, 0.3,
		\gRateMin, 1.7,
		\gRateMax, 1.7,
		\gDurMax, 0.3
	],
	~genGrp
);

~grainGenSynth2 = Synth.new(
	\grainGen,
	[
		\buf, b.bufnum,
		\ptrBus, ~ptrBus,
		\amp, 0.75,
		\ptrLagSec, 0.6,
		\gRateMin, 2,
		\gRateMax, 2,
		\gDurMax, 0.3
	],
	~genGrp
);

~grainGenSynth3 = Synth.new(
	\grainGen,
	[
		\buf, b.bufnum,
		\ptrBus, ~ptrBus,
		\amp, 0.75,
		\ptrLagSec, 1,
		\gRateMin, 3,
		\gRateMax, 3,
		\gDurMax, 0.3
	],
	~genGrp
);

~ptrSynth.set(\t_reset, 1);
)

~grainGenSynth1.set(\gDurMin, 0.02, \gDurMax, 0.08, \gHzMin, 2, \gHzMax, 8, \amp, 0.5, \ptrDev, 0.2);
~grainGenSynth2.set(\gDurMin, 0.02, \gDurMax, 0.08, \gHzMin, 2, \gHzMax, 8, \amp, 0.5, \ptrDev, 0.2);
~grainGenSynth3.set(\gDurMin, 0.02, \gDurMax, 0.08, \gHzMin, 2, \gHzMax, 8, \amp, 0.5, \ptrDev, 0.2);

~grainGenSynth1.set(\gGenType, 1, \gHzMin, 8, \gHzMax, 9);
~grainGenSynth2.set(\gGenType, 1, \gHzMin, 8, \gHzMax, 9);
~grainGenSynth3.set(\gGenType, 1, \gHzMin, 8, \gHzMax, 9);

~grainRecSynth.free

~genGrp.free