//An Introduction to Iteration in SuperCollider

//"do" is a method that is defined for all classes, although its behavior differs from class to class. The user provides a function, and "do" passes its receiver as an argument to the function and evaluates the function.

//"do" is particularly useful with collections (arrays in particular), as it will evaluate its function once per each item in the collection. On each iteration, it will pass as an argument the current item in the collection. For example:

(
[1,3,6,90,4].do({
	arg item;
	item.postln;
});
)

//In the above example, "do" evaluates the function once for each of the five items in the array. In this case, "do" simply posts the item. We also see the original array in the post window, because "do" always returns its receiver -- it's just something that "do" does.

//Here, we iteratively post each item plus 5.
(
[1,3,6,90,4].do({
	arg item;
	(item+5).postln;
});
)

//when used with an integer, let's call it n, "do" interprets n as the array [0, 1, 2, ... n-2, n-1]. For example, this:
(
4.do({
	arg item;
	item.postln;
});
)

//is essentially the same as this:
(
[0,1,2,3].do({
	arg item;
	item.postln;
});
)

//"do" is very useful (and fun) within UGen functions, as it allows us to perform iterative synthesis techniques. Here we have 20 summed stereo sine waves with uniquely chosen frequencies.
(
{
	var sig, sum;
	sum = 0;
	20.do({
		sig = SinOsc.ar({ExpRand(100,1000)}!2);
		sum = sum + sig;
	});
	sum = sum * 0.05;
}.play
)

//We can add a line to our "do" function that modulates the amplitude of each sinusoid component, giving our sound a bubbling/flickering effect:
(
{
	var sig, sum;
	sum = 0;
	20.do({
		sig = SinOsc.ar({ExpRand(100,1000)}!2);
		sig = sig * LFNoise2.kr({ExpRand(4,40)}!2).exprange(0.001,1);
		sum = sum + sig;
	});
	sum = sum * 0.2;
}.play
)


//Here, on each iteration, we generate white noise, and bandpass filter it twice in series. The center frequency is determined by the current iteration number, scaled by an input freuqency argument, thereby corresponding to a sequence of overtones.
(
{
	arg freq=40;
	var sum, sig;
	sum = 0;
	30.do({
		arg i;
			sig = WhiteNoise.ar;
			sig = BPF.ar(sig, (i+1)*freq, ExpRand(0.004, 0.015));
			sig = BPF.ar(sig, (i+1)*freq, {ExpRand(0.004, 0.015)}!2);
			sum = sum + sig;
	});
	sum;
}.play
)


//Using a conditional statement, we will only generate a filtered white noise UGen if the overtone is prime:
(
x = {
	arg freq=40;
	var sum, sig;
	sum = 0;
	30.do({
		arg i;
		if(i.isPrime,
			{
				sig = WhiteNoise.ar;
				sig = BPF.ar(sig, (i+1)*freq, ExpRand(0.004, 0.015));
				sig = BPF.ar(sig, (i+1)*freq, {ExpRand(0.004, 0.015)}!2);
				sum = sum + sig;
			}
		);
	});
	sum;
}.play
)

//We can change the fundamental frequency, since the frequency value is an argument:
x.set(\freq, 30);


//Here's a beefy sum of variable duty saw waves:
(
{
	var sig, sum;
	sum = 0;
	20.do({
		sig = VarSaw.ar({ExpRand(40,41)}!2, Rand(0.0,1.0), ExpRand(0.01, 0.1));
		sig = sig * LFNoise2.kr({ExpRand(1,20)}!2).exprange(0.005,1);
		sum = sum + sig;
	});
	sum = sum * 0.3;
}.play
)

//Last, here's my reverb synth which we've looked at before, although now it probably makes more sense. I'm using sine blips as an input source for ease of demonstration.
(
SynthDef(\reverb_ef, {
	arg	inBus, outBus=0,
		revTime=2.2, mix=0.08, amp=0.5;
	var dry, wet, outsig;
	//dry = In.ar(inBus,2);
	dry = Decay2.ar(Impulse.ar(0.5), 0.02, 0.12, SinOsc.ar(880));
	wet = dry;
	outsig = 0;
	15.do({
		//An allpass filter is a reasonable way to simulate the numerous reflections off hard surfaces, such as walls and ceilings. On each iteration, a new random delay time is chosen.
		wet = AllpassN.ar(wet, 0.06, {Rand(0.001, 0.06)}!2, revTime);
		wet = LPF.ar(wet, 4500);
		outsig = outsig + wet;
	});
	outsig = XFade2.ar(dry, outsig, mix.linlin(0,1,-1,1), amp);
	Out.ar(outBus, outsig);
}).add;
)

x = Synth(\reverb_ef);

//change the mix:
x.set(\mix, 0.8);
x.set(\mix, 0);
x.set(\mix, 0.1);
