//FFT Synthesis in SuperCollider, Part 1

//You'll need to download the SC3 Extensions package in order to run some of the following examples. The package can be found on this page (you'll have to scroll down a bit):

/*
http://supercollider.sourceforge.net/downloads/
*/

s.makeGui.boot;

//sound files used in the following examples (you'll need to change the path names)
c = Buffer.readChannel(s, "/Users/eli/Desktop/piano_trail_01_m.aiff", channels:[0]);
d = Buffer.readChannel(s, "/Users/eli/Desktop/street_noise_2.aiff", channels:[0]);

//Simple Example:
//Here, no spectral synthesis is done. We convert to the frequency domain and immediately resynthesize to the time domain. Additional parameters (hop size, window type) can be specified as additional arguments to FFT.
(
{
	var sig, fsig, out;
	sig = WhiteNoise.ar;
	fsig = FFT(LocalBuf(2048), sig);
	/* FFT synthesis, PV UGens, etc, go here*/
	out = IFFT(fsig);
}.play
)

//Brickwall Filtering using PV_BrickWall:
//If PV_BrickWall's wipe argument is between 0 and 1, bins will be zeroed from bottom to top. If negative, bins will be zeroed from the top down.
(
x = {
	arg bin=0;
	var sig, fsig, out;
	sig = WhiteNoise.ar;
	fsig = FFT(LocalBuf(1024), sig);
	fsig = PV_BrickWall(fsig, bin/512);
	out = IFFT(fsig);
}.play;
)

x.set(\bin, 20); //bottom 20 bins are zeroed
x.set(\bin, -492); //everything except the bottom 20 bins are zeroed

x.free;

//An example of brickwall filtering with a high and low cutoff point
(
x = {
	arg lobin=0, hibin=512;
	var sig, fsig, out;
	sig = WhiteNoise.ar;
	fsig = FFT(LocalBuf(1024), sig);
	fsig = PV_BrickWall(fsig, (512-hibin)/512.neg);
	fsig = PV_BrickWall(fsig, lobin/512);
	out = IFFT(fsig);
}.play;
)

x.set(\lobin, 20, \hibin, 40); //bins 20 through 40
x.set(\lobin, 3, \hibin, 7); //bins 3 through 7
x.set(\lobin, 60, \hibin, 60); //only bin 60

x.free;

//PV UGens overwrite data in the buffer from which they read. Differently named variables can have misleading effects. See the comment in the following example:
(
x = {
	arg lobin=30, hibin=60;
	var sig, fsig, fsig2, fsig3, out;
	sig = WhiteNoise.ar;
	fsig = FFT(LocalBuf(1024), sig);
	fsig2 = PV_BrickWall(fsig, (512-hibin)/512.neg);
	fsig3 = PV_BrickWall(fsig2, lobin/512);

	//If you were to change fsig3 to fsig2, you'd expect to hear all bins below hibin, without the hipass filtering that results from the second PV_BrickWall. Likewise, if you were to replace fsig3 with fsig, you might expect to hear unaltered white noise. However, PV UGens overwrite spectral data in the buffer, so all three f-variables all point to the same buffer, so the result is the same no matter which fsig you resynthesize.
	out = IFFT(fsig3);
}.play;
)

x.free;


//brickwall filtering - an alternate approach with pvcalc
//pvcalc operates on a frequency domain signal (fsig). The user provides a function that accepts two arguments representing an array of amplitudes and an array of phases, and returns a resulting pair of arrays (contained within an array).
(
x = {
	var sig, fsig, out;
	sig = WhiteNoise.ar;
	fsig = FFT(LocalBuf(2048), sig);
	fsig = fsig.pvcalc(
		2048,
		{
			arg mags, phases;
			[mags, phases];
		},
		30,	//low bin
		60,	//high bin
		1	//zero other bins? (1 == true)
	);
	out = IFFT(fsig);
}.play;
)


//PV_BinScramble randomly reorders FFT bins.
(
x = {
	arg wipe=0, width=0, t_trig=1;
	var sig, fsig, out;
	sig = PlayBuf.ar(1, c.bufnum, loop:1);
	fsig = FFT(LocalBuf(2048), sig);
	fsig = PV_BinScramble(fsig, wipe, width/1024, t_trig);
	out = IFFT(fsig);
}.play
)

x.set(\wipe, 0.1, \width, 10, \t_trig, 1);
x.set(\wipe, 0.2, \width, 20, \t_trig, 1);
x.set(\wipe, 0.4, \width, 60, \t_trig, 1);
x.set(\wipe, 0.75, \width, 150, \t_trig, 1);
x.set(\wipe, 1, \width, 300, \t_trig, 1);

x.free;

//FFT morphing
//PV_XFade (a simple crossfade in the frequency domain)
(
x = {
	var sig1, sig2, fsig1, fsig2, fsig, out;
	sig1 = GrainBuf.ar(1, Dust.kr(40), 0.2, c, 1, LFNoise1.kr.range(0,0.9)) * 2;
	sig2 = PlayBuf.ar(1, d, loop:1) * 2;
	fsig1 = FFT(LocalBuf(1024), sig1, 0.5, 0);
	fsig2 = FFT(LocalBuf(1024), sig2, 0.5, 0);
	fsig = PV_XFade(fsig1, fsig2, MouseX.kr.poll);
	out = IFFT(fsig);
}.play;
)


//A detour with PV_XFade: FFT UGens and Multichannel Expansion
//Each signal within a multiple channel signal must be written to a unique buffer. Buffers will not multichannel expand as expected since PV UGens overwrite the data in the buffer from which they read.

//Wrong:
(
x = {
	var sig1, sig2, fsig1, fsig2, fsig, out;
	sig1 = GrainBuf.ar(1, Dust.kr(40!2), 0.2, c, 1, LFNoise1.kr.range(0,0.9)) * 2;
	sig2 = PlayBuf.ar(1!2, d, loop:1) * 2;
	fsig1 = FFT(LocalBuf(1024), sig1, 0.5, 0); //these buffers should be 2-channel
	fsig2 = FFT(LocalBuf(1024), sig2, 0.5, 0);
	fsig = PV_XFade(fsig1, fsig2, MouseX.kr);
	out = IFFT(fsig);
}.play;
)

//Right:
(
x = {
	var sig1, sig2, fsig1, fsig2, fsig, out;
	sig1 = GrainBuf.ar(1, Dust.kr(40!2), 0.2, c, 1, LFNoise1.kr.range(0,0.9)) * 2;
	sig2 = PlayBuf.ar(1!2, d, loop:1) * 2;
	fsig1 = FFT({LocalBuf(1024)}!2, sig1, 0.5, 0); //stereo signal gets a stereo buffer
	fsig2 = FFT({LocalBuf(1024)}!2, sig2, 0.5, 0);
	fsig = PV_XFade(fsig1, fsig2, MouseX.kr);
	out = IFFT(fsig);
}.play;
)


//PV_Morph - like PV_XFade, but with some cross-synthesis between signals
(
x = {
	var sig1, sig2, fsig1, fsig2, fsig, out;
	sig1 = GrainBuf.ar(1, Dust.kr(40!2), 0.2, c, 1, LFNoise1.kr.range(0,0.9)) * 2;
	sig2 = PlayBuf.ar(1!2, d, loop:1) * 2;
	fsig1 = FFT({LocalBuf(1024)}!2, sig1, 0.5, 0);
	fsig2 = FFT({LocalBuf(1024)}!2, sig2, 0.5, 0);
	fsig = PV_Morph(fsig1, fsig2, MouseX.kr);
	out = IFFT(fsig);
}.play;
)


//PV_BinWipe (replace bins from fsigA to fsigB)
//PV_SoftWipe is like PV_BinWipe but interpolates between bins.
(
x = {
	var sig1, sig2, fsig1, fsig2, fsig, out;
	sig1 = GrainBuf.ar(1, Dust.kr(40!2), 0.2, c, 1, LFNoise1.kr.range(0,0.9));
	sig2 = PlayBuf.ar(1!2, d, loop:1);
	fsig1 = FFT({LocalBuf(1024)}!2, sig1, 0.5, 0);
	fsig2 = FFT({LocalBuf(1024)}!2, sig2, 0.5, 0);
	fsig = PV_BinWipe(fsig2, fsig1, MouseX.kr(0.01,1,1).poll);
	out = IFFT(fsig);
}.play;
)

//More nuanced control of two FFT signals with pvcalc2:
//This example uses horizonal mouse control to take amplitudes from one frequency domain signal and combine them with the phases from another frequency domain signal. The process is reversed as the mouse moves from one side of the screen to the other.
(
x = {
	var sig1, sig2, fsig1, fsig2, fsig, out;
	sig1 = GrainBuf.ar(1, Dust.kr(40), 0.2, c, 1, LFNoise1.kr.range(0,0.9));
	sig2 = PlayBuf.ar(1, d, loop:1);
	fsig1 = FFT(LocalBuf(1024), sig1, 0.5, 0);
	fsig2 = FFT(LocalBuf(1024), sig2, 0.5, 0);
	fsig = fsig1.pvcalc2(
		fsig2,
		1024,
		{
			arg mags1, phs1, mags2, phs2;
			var blend, newmags, newphs;
			blend = MouseX.kr(0,1).poll;
			newmags = (mags1 * blend) + (mags2 * (1-blend));
			newphs = (phs1 * (1-blend)) + (phs2 * blend);
			[newmags, newphs]
		},
		0,
		1024,
		0
	);
	out = IFFT(fsig)!2;
}.play;
)