//FFT Synthesis in SuperCollider, Part 2

s.makeGui.boot;

//Frequency Domain Time Stretching:

//a String representing a path to an audio file - change this as necessary
~path = "/Users/eli/Desktop/084gNostrandV2-4.aiff";

//a buffer containing a time domain audio signal
~audioBuf = Buffer.readChannel(s, ~path, channels:[0]);

//a buffer which will contain frequency domain information for the above audio signal. 'calcPVRecSize' interprets its numerical receiver (in this case, ~audioBuf.duration) as the duration of a sound file and will return the appropriate number of samples to hold FFT data for a buffer with the given frame size and hop size.
~fftBuf = Buffer.alloc(s, ~audioBuf.duration.calcPVRecSize(1024, 0.25));

//a SynthDef which writes spectral data to a buffer
(
SynthDef(\pvrec, {
	arg recbuf, audiobuf;
	var sig, fsig;

	//Line will free the Synth when recording has finished
	Line.kr(1,1,BufDur.kr(audiobuf), doneAction:2);

	//the signal to record
	sig = PlayBuf.ar(1, audiobuf);

	//convert to frequency domain
	fsig = FFT(LocalBuf(1024), sig, 0.25, 1);

	//write frequency domain data to a buffer
	fsig = PV_RecordBuf(fsig, recbuf, 0, 1, 0, 0.25, 1);
}).add;
)

//instantiate the FFT record synth providing the appropriate bufnums as arguments
a = Synth(\pvrec, [\recbuf, ~fftBuf.bufnum, \audiobuf, ~audioBuf.bufnum]);

//you can write your FFT buffer to your hard disk for use in other SC sessions
~fftBuf.write(~path++".scpv", "aiff", "float");

//the counterpart to SynthDef(\pvrec), this SynthDef uses PV_BufRd to read frequency domain data stored in a buffer according to an audio rate index, much like the time domain version of this UGen, BufRd.
(
SynthDef(\pvplay, {
	arg out=0, recbuf, audiobuf, rate=1;
	var sig, fsig, index;
	index = Phasor.ar(0,BufFrames.ir(audiobuf).reciprocal*rate,0.0,1.0,0);
	fsig = PV_BufRd(LocalBuf(1024), recbuf, index);
	Out.ar(out, IFFT(fsig, 1) * 0.75!2);
}).add;
)

x = Synth(\pvplay, [\recbuf, ~fftBuf, \audiobuf, ~audioBuf])

x.set(\rate, 1.2);
x.free;

//If you quit and reopen SC, here's how you can read your FFT data to a Buffer without reinstantiating a pvrec Synth.
~fftBuf = Buffer.read(s, "/Users/eli/Desktop/084gNostrandV2-4.aiff.scpv");




//Onset Detection:
//PV_HainsworthFoote is a frequency domain UGen that analyzes spectral data and outputs audio rate impulses in the time domain - very fun and useful.

//simply listen to a sine blip when an onset is detected - you might have to tweak the threshold parameter.
(
x = {
	var source, fsig, detect;
	source = SoundIn.ar(0);
	fsig = FFT(LocalBuf(2048), source);
	detect = PV_HainsworthFoote.ar(fsig, 1.0, 0.0, 2, 0.05);
	Decay2.ar(detect, 0.01, 0.02, SinOsc.ar(440,0,0.2));
}.play
)

x.free;

//using onsets to reset buffer playback
(
x = {
	var source, fsig, detect;
	source = SoundIn.ar(0);
	fsig = FFT(LocalBuf(2048), source);
	detect = PV_HainsworthFoote.ar(fsig, 1.0, 0.0, 2, 0.05);
	PlayBuf.ar(1!2, ~audioBuf, 1, detect, 0, 1);
}.play
)

x.free;

//using onsets to randomize pointer position in a buffer
(
x = {
	var source, fsig, detect, phs;
	source = SoundIn.ar(0);
	fsig = FFT(LocalBuf(2048), source);
	detect = PV_HainsworthFoote.ar(fsig, 1.0, 0.0, 2, 0.05);
	phs = Phasor.ar(0, 1, 0, ~audioBuf.numFrames, 0);
	phs = phs + TRand.ar(0, ~audioBuf.numFrames, detect);
	phs = phs % ~audioBuf.numFrames;
	BufRd.ar(1!2, ~audioBuf, phs, 1, 2);
}.play
)

x.free;

//using onsets to trigger changes in playback rate
(
x = {
	var source, fsig, detect, phs, rate;
	source = SoundIn.ar(0);
	fsig = FFT(LocalBuf(2048), source);
	detect = PV_HainsworthFoote.ar(fsig, 1.0, 0.0, 2, 0.05);
	rate = TRand.kr(0.8,1.25,detect);
	rate = Lag.kr(rate, 0.5);
	phs = Phasor.ar(0, rate, 0, ~audioBuf.numFrames, 0);
	BufRd.ar(1!2, ~audioBuf, phs, 1, 2);
}.play
)

x.free;