//Xbox Kinect & SuperCollider via Synapse and OSC Messaging

/*
See the following link for more information:
http://synapsekinect.tumblr.com/post/6307752257/maxmsp-jitter
*/

//You can download Synapse here:
/*
http://synapsekinect.tumblr.com/post/6305020721/download
*/

//If you have an Xbox Kinect, make sure it's plugged into the wall and to your computer's USB hub, and also make sure the Synapse application is up and running.

////////////////////////////////////////////
//****************************************//
//****************************************//
//****************************************//
////////////////////////////////////////////

s.makeGui.boot;

//create a NetAddr representing the Kinect (an IP address and OSC port number). Synapse receives messages on port 12346. The IP address 127.0.0.1 is a loopback address referring to your own computer.
~synapse = NetAddr.new("127.0.0.1", 12346);


////////////////////////////////////////////
//****************************************//
//****************************************//
//****************************************//
////////////////////////////////////////////

//As simple as it gets:

//Every two seconds, send an OSC message to the Kinect, letting it know that you want to keep tracking the position of a joint. In this case, it's the position of the right hand.

(
~trackBody = {
	inf.do{
		~synapse.sendMsg("/righthand_trackjointpos", 1).postln;
		2.wait;
	}
}.fork;
)

//Immediately above, the message "/<joint>_trackjointpos" is accompanied by an integer:
//1 -> joint position relative to torso, in millimeters.
//2 -> joint position in world space, in millimeters.
//3 -> joint position on screen, in pixels.

//In reponse to receiving certain OSC messages, the Kinect will send certain OSC messages on ports 12345 and 12347. Create an OSCdef in SuperCollider in order to receive these messages and evaluate a function using data from the OSC message as arguments. In this case, we're just posting the raw OSC message and its relevant data:
(
OSCdef.new(
	\rh_xyz,
	{
		|msg, time, addr, recvPort|
		[msg, time, addr, recvPort].postln;
	},
	"/righthand_pos_body", nil, 12345
)
)

//Stop sending and receiving OSC messages
(
~trackBody.stop;
OSCdef.freeAll;
)

////////////////////////////////////////////
//****************************************//
//****************************************//
//****************************************//
////////////////////////////////////////////

//Suppose we don't want to be receiving joint data at such a high frequency. We can choose our own rate of receiving data by selectively enabling and disabling the OSC responder:

(
~trackBody = {
	inf.do{

		//whenever we query Synapse for joint data, enable the responder:
		OSCdef(\rh_xyz).enable;

		~synapse.sendMsg("/righthand_trackjointpos", 1).postln;

		//then wait for the desired amount of time
		0.2.wait;
	}
}.fork;
)

(
OSCdef.new(
	\rh_xyz,
	{
		|msg, time, addr, recvPort|
		[msg, time, addr, recvPort].postln;

		//at the very end of our OSC function, the responder will disable itself. Therefore, the function will be evaluated once, and won't be evaluated again until our ~trackBody Routine sends another query message to Synapse.
		OSCdef(\rh_xyz).disable;
	},
	"/righthand_pos_body", nil, 12345

).disable; //upon instantiation, the OSC responder is disabled by default.
)


//Stop sending and receiving OSC messages
(
~trackBody.stop;
OSCdef.freeAll;
)

////////////////////////////////////////////
//****************************************//
//****************************************//
//****************************************//
////////////////////////////////////////////

//Tracking the position of a joint (more sophisticated):

//We'll use Tdef to create a Task that sends OSC messages to the Kinect, instead of forking a Function to return a Routine. Like OSCdef, MIDIdef, SynthDef, Pdefn, etc, Tdef keeps a reference to an object (in this case, a Task), and allows us to swap one particular Task for another in real-time, as the Tdef is playing.

(
Tdef(\trackBody, {
	{
		OSCdef(\rh_xyz).enable;
		~synapse.sendMsg("/righthand_trackjointpos", 2);
		"querying joint data".postln;
		0.2.wait;
	}.loop
}).play
)

//This OSCdef will post Kinect data (in this case, for the the position of the right hand relative to the body) in a more meaningful format.
(
OSCdef.new(
	\rh_xyz,
	{
		|msg, time, addr, recvPort|
		("x pos: "++msg[1].round(0.01)).postln;
		("y pos: "++msg[2].round(0.01)).postln;
		("z pos: "++msg[3].round(0.01)).postln;
		"".postln;
		OSCdef(\rh_xyz).disable;
	},
	"/righthand_pos_world", nil, 12345
).disable
)

//Since we're using a Tdef, we can change the frequency of data receipt in real-time, by redefining the Tdef:
(
Tdef(\trackBody, {
	{
		OSCdef(\rh_xyz).enable;
		~synapse.sendMsg("/righthand_trackjointpos", 1);
		"querying joint data".postln;
		0.05.wait; //receive joint data 20x per second
	}.loop
}); //Notice that we don't have to append the "play" message, since the Tdef is already playing
)

//Stop sending and receiving OSC messages
(
Tdef(\trackBody).stop;
OSCdef.freeAll;
)

////////////////////////////////////////////
//****************************************//
//****************************************//
//****************************************//
////////////////////////////////////////////

//Tracking the positions of two (or more) joints with the 'sendBundle' method

(
Tdef(\trackBody, {
	{
		OSCdef(\rh_xyz).enable;
		OSCdef(\lh_xyz).enable;
		~synapse.sendBundle(
			0.0,
			["/righthand_trackjointpos", 1],
			["/lefthand_trackjointpos", 1]
		);
		"querying LH & RH data".postln;
		0.3.wait;
	}.loop
}).play
)

//This OSCdef will post Kinect data for the right hand,
(
OSCdef.new(
	\rh_xyz,
	{
		|msg, time, addr, recvPort|
		("rh x pos: "++msg[1].round(0.01)).postln;
		("rh y pos: "++msg[2].round(0.01)).postln;
		("rh z pos: "++msg[3].round(0.01)).postln;
		"".postln;
		OSCdef(\rh_xyz).disable;
	},
	"/righthand_pos_body", nil, 12345
).disable;

//and this OSCdef posts data for the left hand.
OSCdef.new(
	\lh_xyz,
	{
		|msg, time, addr, recvPort|
		("lh x pos: "++msg[1].round(0.01)).postln;
		("lh y pos: "++msg[2].round(0.01)).postln;
		("lh z pos: "++msg[3].round(0.01)).postln;
		"".postln;
		OSCdef(\lh_xyz).disable;
	},
	"/lefthand_pos_body", nil, 12345
).disable;
)

//Stop sending and receiving OSC messages
(
Tdef(\trackBody).stop;
OSCdef.freeAll;
)

////////////////////////////////////////////
//****************************************//
//****************************************//
//****************************************//
////////////////////////////////////////////

//SynthDef from Assignment 1: old version
(
SynthDef(\ethereal_old, {
	arg gate=1, fund=50, out=0;
	var sig, bpfHz, amp, env;
	env = EnvGen.kr(Env.adsr(15,0.1,1,15),gate,doneAction:2);
	amp = LFNoise1.kr({exprand(0.1,0.8)}!12).exprange(0.03,1);

	//center freqs are chosen autonomously with LFNoise0
	bpfHz = LFNoise0.kr({[0.05,0.1,0.2,0.25,1/3].choose}!12).exprange(fund*2,fund*15);
	bpfHz = bpfHz.round(fund);

	sig = PinkNoise.ar(1!12);
	sig = BPF.ar(sig, bpfHz, 400.reciprocal) * amp;
	sig = SplayAz.ar(4, sig);
	sig = BLowShelf.ar(sig, 200, 0.5, 6) * 16 * env;
	Out.ar(out, sig);
}).add;
)

x = Synth(\ethereal_old);
x.set(\gate, 0);

//new version
(
SynthDef(\ethereal, {
	arg gate=1, fund=50, t_bpf=0, detune=0, out=0;
	var sig, bpfHz, amp, env;
	env = EnvGen.kr(Env.adsr(15,0.1,1,15),gate,doneAction:2);
	amp = LFNoise1.kr({exprand(0.1,0.8)}!12).exprange(0.03,1);

	//center freq changes are triggered manually with 't_bpf'
	bpfHz = TChoose.kr(t_bpf!12, (2,3..15)*fund);

	//and detuned according to the 'detune' argument
	bpfHz = bpfHz + LFNoise1.kr(1!12).bipolar(detune);

	sig = PinkNoise.ar(1!12);
	sig = BPF.ar(sig, bpfHz, 400.reciprocal) * amp;
	sig = SplayAz.ar(4, sig);
	sig = BLowShelf.ar(sig, 200, 0.5, 6) * 16 * env;
	Out.ar(out, sig);
}).add;
)

x = Synth(\ethereal);

x.set(\t_bpf, 1); //run this line several times

x.set(\detune, rrand(0,50).postln);

x.set(\gate, 0);

////////////////////////////////////////////

x = Synth(\ethereal);

(
Tdef(\trackBody, {
	{
		~synapse.sendBundle(0.0, ["/righthand_trackjointpos", 1], ["/lefthand_trackjointpos", 1]);
		"querying LH & RH data".postln;
		2.wait;
	}.loop
}).play
)

(
~lhx = 0; ~lhxDif = 0;
~lhy = 0; ~lhyDif = 0;
OSCdef.new(
	\lh_xyz,
	{
		|msg, time, addr, recvPort|
		("lh x pos: "++msg[1].round(0.01)).postln;
		("lh y pos: "++msg[2].round(0.01)).postln;
		~lhxDif = abs(~lhx - msg[1]);
		~lhyDif = abs(~lhy - msg[2]);
		~lhxDif = ~lhxDif.linlin(0,900,0,350);
		~lhyDif = ~lhyDif.linlin(0,900,0,350);

		//detune the center freqs according to the difference between the left hand's xy position between the current frame and the previous frame
		x.set(\detune, max(~lhxDif, ~lhyDif));

		~lhx = msg[1];
		~lhy = msg[2];
	},
	"/lefthand_pos_body", nil, 12345
);

OSCdef.new(
	\rh_hit,
	{
		|msg, time, addr, recvPort|
		msg.postln;
		"".postln;
		//retrigger new center freqs whenever an "up" hit is received by the right hand
		if(msg[1] == \up, {x.set(\t_bpf, 1)});
	},
	"/righthand", nil, 12345
);
)


//Stop sending and receiving OSC messages
(
x.free;
Tdef(\trackBody).stop;
OSCdef.freeAll;
)

////////////////////////////////////////////

//In this version of the ethereal SynthDef, the bpf trigger is no longer a trigger argument. Instead it is a normal control argument that needs to be manually reset to a value at or below zero before it can be retriggered.
(
SynthDef(\ethereal2, {
	arg gate=1, fund=50, bpf=0, detune=0, out=0;
	var sig, bpfHz, amp, env;
	env = EnvGen.kr(Env.adsr(15,0.1,1,15),gate,doneAction:2);
	amp = LFNoise1.kr({exprand(0.1,0.8)}!12).exprange(0.03,1);
	bpfHz = TChoose.kr(bpf!12, (2,3..15)*fund);
	bpfHz = bpfHz + LFNoise1.kr(1!12).bipolar(detune);
	sig = PinkNoise.ar(1!12);
	sig = BPF.ar(sig, bpfHz, 400.reciprocal) * amp;
	sig = SplayAz.ar(4, sig);
	sig = BLowShelf.ar(sig, 200, 0.5, 6) * 16 * env;
	Out.ar(out, sig);
}).add;
)

x = Synth(\ethereal2);

(
Tdef(\trackBody, {
	{
		~synapse.sendBundle(0.0, ["/righthand_trackjointpos", 2], ["/lefthand_trackjointpos", 1]);
		"querying LH & RH data".postln;
		2.wait;
	}.loop
}).play
)

(
~lhx = 0; ~lhxDif = 0;
~lhy = 0; ~lhyDif = 0;
OSCdef.new(
	\lh_xyz,
	{
		|msg, time, addr, recvPort|
		("lh x pos: "++msg[1].round(0.01)).postln;
		("lh y pos: "++msg[2].round(0.01)).postln;
		~lhxDif = abs(~lhx - msg[1]);
		~lhyDif = abs(~lhy - msg[2]);
		~lhxDif = ~lhxDif.linlin(0,900,0,350);
		~lhyDif = ~lhyDif.linlin(0,900,0,350);
		x.set(\detune, max(~lhxDif, ~lhyDif));
		~lhx = msg[1];
		~lhy = msg[2];
	},
	"/lefthand_pos_body", nil, 12345
);

OSCdef.new(
	\rh_xyz,
	{
		|msg, time, addr, recvPort|
		msg[2].postln;
		"".postln;

		//instead of relying on "hits" from Synapse, we're thresholding the y position of the right hand. When it exceeds 600 mm, a trigger is generated. When it drops below 0 mm, the argument is set to 0 and can be retriggered.
		if(msg[2] >= 600, {x.set(\bpf, 1)});
		if(msg[2] < 0, {x.set(\bpf, 0)});
	},
	"/righthand_pos_world", nil, 12345
);
)

//Stop sending and receiving OSC messages
(
x.free;
Tdef(\trackBody).stop;
OSCdef.freeAll;
)