//Routing Audio Between SuperCollider and Other Audio-Aware Applications

//There are numerous applications that can be used to internally route audio between applications on your computer, such as ReWire, Jack, and Soundflower. I am most familiar with Soundflower, which is capable of routing 64 discrete channels of audio and freely available here:

/*
http://cycling74.com/soundflower-landing-page/
*/

//Once you have installed Soundflower and restarted your computer, Soundflower will appear as an audio hardware device in System Preferences >> Sound (on Mac OS X). Technically it appears as two devices: Soundflower (2ch) and Soundflower (64ch).

//In SuperCollider (which you may need to relaunch at this point), you can view the list of available audio devices using the following code:

ServerOptions.devices;

//Since this code returns an Array of names (Strings) of audio devices, we can refer to an individual device using the square bracket "at" syntax (or the actual "at" method):

ServerOptions.devices[0];

//Alternatively, you can specify a device by providing a String yourself, but you'll have to make sure you've spelled it correctly. You can set SC's output device like so:

s.options.outDevice = "Soundflower (64ch)";

//Since the 64 channel version of Soundflower has 64 discrete outputs, we should also change the 'numOutputBusChannels' attribute of the server's options to be equal to 64, for the sake of consistency:

s.options.numOutputBusChannels = 64;

//For these changes to take effect, we need to reboot the server:

s.reboot;

//Now the following lines should return "Soundflower (64ch)" and "64", respectively:

s.options.outDevice;
s.options.numOutputBusChannels;

//Like SuperCollider, any other audio-aware application (such as a DAW, MaxMSP, etc) is now capable of using Soundflower (64ch) as an input/output device. In order to have another application recieve SC's audio output, set that application's input device to Soundflower (64ch).

//Now we'll instantiate a quick Synth that generates short sine blips every second. It will output audio on Soundflower 64ch's lowest hardware output bus, bus zero.

(
x = {
	var sig;
	sig = Decay2.ar(Impulse.ar(1), 0.01, 0.1, SinOsc.ar(440,0,0.5));
	Out.ar(0, sig);
}.play
)

//Have your receiving application listen for audio on the lowest numbered input bus. Remember that whereas SC starts counting bus indices at zero, most other applications start counting at one. You should be receiving sine blips from SC in real-time. Once you've ensured that audio is being routed, the sky's the limit.

//Here's an example of routing audio within SuperCollider, and then routing externally to another application. First, allocate an audio bus. Because we've specified 64 hardware output channels, this method of bus allocation will bypass all of these "dedicated" busses. SuperCollider will also bypass the dedicated hardware input busses. By default, SC assumes 8 hardware input channels. We haven't messed with this value, and in this case it doesn't matter, since we're only outputting audio to Soundflower.

//Therefore, when we allocate this bus, it will begin with the first "private" audio bus, with index 64 + 8 = 72. Verify this in the post window.

b = Bus.audio(s, 1);

//Now we'll instantiate a reverb Synth that reads audio from bus b and applies reverb,
(
y = {
	var sig;
	sig = In.ar(b, 1);
	sig = GVerb.ar(sig, 99, 3) * 0.3;
	sig = Out.ar(0, sig);
}.play;
)

//and a sound source, our sine blips, that output to bus b.
(
x = {
	var sig;
	sig = Decay2.ar(Impulse.ar(1), 0.01, 0.1, SinOsc.ar(440,0,0.5));
	Out.ar(b, sig);
}.play;
)

//Keep in mind that GVerb also happens to turn a mono signal into a stereo signal, so we're actually sending audio to busses 0 and 1 on Soundflower. You should be able to receive this stereo reverb signal using other applications by listening to the correct busses.

x.free;
y.free;

//You can create a virtual device that is an aggregate of several discrete devices, in order to route audio back and forth internally in addition to sending audio to your speakers. In Applications >> Utilities >> Audio MIDI Setup (Mac OS X), view the Audio Devices window if it is not already visible using the Window drop-down menu. Click the "+" icon in the lower left corner to add an aggregate device, and check the boxes of the discrete devices that you'd like to add to the aggregate (thanks very much to Yago de Quay for this tip).