//Drawing & Animation using the Pen class

//The 'Pen' class allows the user to draw and create animation on GUI windows. Pen is unique in that it has no instances and no instance methods. While it is necessary to create instances of other classes (SinOsc.ar, Window.new, Buffer.alloc, etc), all messages to Pen are sent to the class itself. Pen behaves as an omnipresent hand-of-god pen, operating perpetually invisibly. Through Pen's class methods, the user tells Pen to move to a certain pixel on a window, draw a line between two points, draw a rectangle, draw an arc, change the color of the drawn lines, change the width of drawn lines, etc.

//In almost all cases in which Pen is being used, you will create a Window with a single GUI object - a UserView. Unlike a Button, a PopUpMenu, a TextView, a Slider, etc, a UserView does not have any direct functionality, but instead serves as a palette for drawing and animating with Pen. Set the following Window aside, we'll use it for multiple examples below:

(
w = Window.new.front;
u = UserView.new(w, w.view.bounds).background_(Color.white);
)

//UserView has a special instance method that other GUI Views do not have: 'drawFunc'. Using this method, you can define a function to be evaluated whenever the UserView or Window is refreshed (using the 'refresh' method). Pen commands can ONLY be used within a drawFunc.

//By default, the UserView's drawFunc is nil.
u.drawFunc;

//But we can define a function ourselves. One of the simplest methods for Pen is 'line'. We specify two points (x@y), corresponding to pixels on the UserView, and Pen draws a linebetween them.

(
u.drawFunc_({
	Pen.line(200@200, 300@350);
});

u.refresh;
)

//But wait a minute, why don't we see anything? SC isn't even posting an error message. Here's why: The pen won't actually draw any lines until you explicitly tell it to using the 'stroke' command.

(
u.drawFunc_({
	Pen.line(200@200, 300@350);
	Pen.stroke;
});

w.refresh; //Refreshing the parent Window (instead of the UserView) is equivalent.
)

//And there's our line. We've defined a function with constant numerical values, so multiple refreshes won't actually change the appearance of our UserView. But with a little randomness...

(
u.drawFunc_({
	Pen.line(

		//random start point
		rrand(0,u.bounds.width)@rrand(0,u.bounds.height),

		//random end point
		rrand(0,u.bounds.width)@rrand(0,u.bounds.height));

	Pen.stroke;
});

u.refresh;
)

//Now, on every refresh, the function will be re-evaluated, randomly choose start and end points, and connect them with a line. We can create a Routine to autonomously refresh the UserView. Remember that when controlling GUIs with Routines, the Routine must be played on the AppClock.

(
~refreshRoutine = Routine({
	inf.do{
		u.refresh;
		(1/30).wait; //30 frames per second
	};
}).play(AppClock);
)

~refreshRoutine.stop;

//There's an even better way to animate your Pen commands using UserView's 'frameRate' and 'animate' methods. When 'animate' is set to true, SC calls 'refresh' on the receiving UserView at a rate specified by 'frame rate'. Very convenient!

//start animating at 30 fps
(
u.frameRate = 30;
u.animate = true;
)

//stop animating
u.animate = false;





//Be sure to check out the Pen help file for more drawing techniques. Here are some more complex examples:

//a wacky line (very similar to the previous example)
(
w = Window.new("animation", Rect(128,128,1024,768)).front;
u = UserView.new(w, w.view.bounds).background_(Color.white);
w.onClose_({u.animate = false});
u.drawFunc_({
	Pen.line(
		rrand(0,w.view.bounds.width)@rrand(0,w.view.bounds.height),
		rrand(0,w.view.bounds.width)@rrand(0,w.view.bounds.height)
	);
	Pen.width = 3;
	Pen.stroke;
});
u.frameRate = 30;
u.animate = true;
)



//a trembling line that moves with Brownian motion
(
var startX, startY, endX, endY;

w = Window.new("animation", Rect(128,128,1024,768)).front;
u = UserView.new(w, w.view.bounds).background_(Color.white);
w.onClose_({u.animate = false});

startX = rrand(0,w.view.bounds.width);
startY = rrand(0,w.view.bounds.height);
endX = rrand(0,w.view.bounds.width);
endY = rrand(0,w.view.bounds.height);

u.drawFunc_({
	Pen.line(startX@startY,endX@endY);
	Pen.width = 3;
	Pen.stroke;
	startX = (startX + (-5..5).choose).clip(0,w.view.bounds.width);
	startY = (startY + (-5..5).choose).clip(0,w.view.bounds.height);
	endX = (endX + (-5..5).choose).clip(0,w.view.bounds.width);
	endY = (endY + (-5..5).choose).clip(0,w.view.bounds.height);
});
u.frameRate = 30;
u.animate = true;
)




//an arbitrary number of trembling lines with Brownian motion
(
var numLines=50;
var startX, startY, endX, endY;

w = Window.new("animation", Rect(128,128,1024,768)).front;
u = UserView.new(w, w.view.bounds)
.background_(Color.white);
w.onClose_({u.animate = false});

startX = Array.fill(numLines, {rrand(0,w.view.bounds.width)});
startY = Array.fill(numLines, {rrand(0,w.view.bounds.height)});
endX = Array.fill(numLines, {rrand(0,w.view.bounds.width)});
endY = Array.fill(numLines, {rrand(0,w.view.bounds.height)});

u.drawFunc_({
	numLines.do{
		|n|
		Pen.line(startX[n]@startY[n],endX[n]@endY[n]);
		Pen.width = 3;
		Pen.stroke;
		startX[n] = (startX[n] + (-5..5).choose).clip(0,w.view.bounds.width);
		startY[n] = (startY[n] + (-5..5).choose).clip(0,w.view.bounds.height);
		endX[n] = (endX[n] + (-5..5).choose).clip(0,w.view.bounds.width);
		endY[n] = (endY[n] + (-5..5).choose).clip(0,w.view.bounds.height);
	}
});
u.frameRate = 30;
u.animate = true;
)





//flashy/seizure inducing
(
var numLines=30;
var startX, startY, endX, endY;

w = Window.new("animation", Rect(128,128,1024,768)).front;
u = UserView.new(w, w.view.bounds)
.background_(Color.white);
w.onClose_({u.animate = false});

startX = Array.fill(numLines, {rrand(0,w.view.bounds.width)});
startY = Array.fill(numLines, {rrand(0,w.view.bounds.height)});
endX = Array.fill(numLines, {rrand(0,w.view.bounds.width)});
endY = Array.fill(numLines, {rrand(0,w.view.bounds.height)});

u.drawFunc_({
	numLines.do{
		|n|
		Pen.line(startX[n]@startY[n],endX[n]@endY[n]);
		Pen.width = 10;
		Pen.strokeColor = Color.rand;
		Pen.stroke;
		startX[n] = (startX[n] + (-15..15).choose).clip(0,w.view.bounds.width);
		startY[n] = (startY[n] + (-15..15).choose).clip(0,w.view.bounds.height);
		endX[n] = (endX[n] + (-15..15).choose).clip(0,w.view.bounds.width);
		endY[n] = (endY[n] + (-15..15).choose).clip(0,w.view.bounds.height);
	}
});
u.frameRate = 30;
u.animate = true;
)



//each line is given a constant, unique color
(
var numLines=30;
var startX, startY, endX, endY;
var colors;

colors = Array.fill(numLines, {Color.rand});

w = Window.new("animation", Rect(128,128,1024,768)).front;
u = UserView.new(w, w.view.bounds)
.background_(Color.white);
w.onClose_({u.animate = false});

startX = Array.fill(numLines, {rrand(0,w.view.bounds.width)});
startY = Array.fill(numLines, {rrand(0,w.view.bounds.height)});
endX = Array.fill(numLines, {rrand(0,w.view.bounds.width)});
endY = Array.fill(numLines, {rrand(0,w.view.bounds.height)});

u.drawFunc_({
	numLines.do{
		|n|
		Pen.capStyle = 1; //rounded line endpoints
		Pen.line(startX[n]@startY[n],endX[n]@endY[n]);
		Pen.width = 10;
		Pen.strokeColor = colors[n];
		Pen.stroke;
		startX[n] = (startX[n] + (-5..5).choose).clip(0,w.view.bounds.width);
		startY[n] = (startY[n] + (-5..5).choose).clip(0,w.view.bounds.height);
		endX[n] = (endX[n] + (-5..5).choose).clip(0,w.view.bounds.width);
		endY[n] = (endY[n] + (-5..5).choose).clip(0,w.view.bounds.height);
	}
});
u.frameRate = 30;
u.animate = true;
)

//Animation and Sound

//boot the server
s.boot;

//free all Buffers, just to be safe;
Buffer.freeAll;

//This example is more rewarding with a long enjoyable sound file with substantial variation in amplitude. I'm using cueSoundFile to stream a Ratatat track from my hard drive, but you can use whatever you like.
b = Buffer.cueSoundFile(s, "/Users/eli/Sounds/scaudio/brazil.aiff", 0, 2);

//Define a UGen function to control the animation
(
~jam = {
	arg amp=1;
	var sig, ampTrack, sendTrig;

	//stream the sound file
	sig = DiskIn.ar(2, b.bufnum) * amp;

	//track the amplitude
	ampTrack = Amplitude.kr(BPF.ar(sig,100,0.2));

	//smooth out the signal a bit
	ampTrack = Lag.kr(ampTrack, 0.05);

	//We'll query the tracked amplitude 50 times per second
	sendTrig = Impulse.kr(50);

	//send the amplitude back to the language so it can be incorporated into the animation
	SendReply.kr(sendTrig, '/level', ampTrack);

	//an EnvGate for ease of fadeout
	sig = sig * EnvGate.new;

	Out.ar(0, sig * 0.4);
};
)

//Make an OSCdef to receive the amplitude data from the server
(
OSCdef.new(\getLevel, {
	arg msg, time, addr, recvPort;

	//I'm going to use the amplitude of the signal to control Pen's width
	~penWidth = msg[3].linlin(0.05, 0.5, 1, 200);

	}, '/level'
);
)

//Create the Window, UserView, and drawFunc
(
var uv, color;

color = Color.rand;
~penWidth = 3;

w = Window.new("animation", Rect(128,128,650,550)).front;
uv = UserView.new(w, w.view.bounds)
.background_(Color.white);
w.onClose_({uv.animate = false});

uv.drawFunc_({
	|n|
	Pen.line(200@200, 500@400);
	Pen.width = ~penWidth;
	Pen.strokeColor = color;
	Pen.stroke;
});
uv.frameRate = 60;
uv.animate = true;
)

//Move your Window where you can see it well, and instantiate a Synth from our UGen function.
~playJam = ~jam.play;

//The amplitude of the signal is controlling the animation in real-time, so adjustments to the Synth will be mirrored in the animation:

//amplitude adjustment
~playJam.set(\amp, 0.25);

//fade out
~playJam.set(\fadeTime, 10, \gate, 0);

//close the window
w.close;